#ifndef lint
static char const 
yyrcsid[] = "$FreeBSD: src/usr.bin/yacc/skeleton.c,v 1.28.2.1 2001/07/19 05:46:39 peter Exp $";
#endif
#include <stdlib.h>
#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYLEX yylex()
#define YYEMPTY -1
#define yyclearin (yychar=(YYEMPTY))
#define yyerrok (yyerrflag=0)
#define YYRECOVERING() (yyerrflag!=0)
#if defined(__cplusplus) || __STDC__
static int yygrowstack(void);
#else
static int yygrowstack();
#endif
#define YYPREFIX "yy"
#line 2 "ns_parser.y"
#if !defined(lint) && !defined(SABER)
static char rcsid[] = "$Id: ns_parser.y,v 8.88 2003/10/16 00:13:35 marka Exp $";
#endif /* not lint */

/*
 * Copyright (c) 1996-2000 by Internet Software Consortium.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE CONSORTIUM DISCLAIMS
 * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL INTERNET SOFTWARE
 * CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

/* Global C stuff goes here. */

#include "port_before.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/un.h>

#include <netinet/in.h>
#include <arpa/nameser.h>
#include <arpa/inet.h>

#include <ctype.h>
#include <limits.h>
#include <netdb.h>
#include <resolv.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <time.h>

#include <isc/eventlib.h>
#include <isc/logging.h>

#include <isc/dst.h>

#include "port_after.h"

#include "named.h"
#include "ns_parseutil.h"
#include "ns_lexer.h"

#define SYM_ZONE	0x010000
#define SYM_SERVER	0x020000
#define SYM_KEY		0x030000
#define SYM_ACL		0x040000
#define SYM_CHANNEL	0x050000
#define SYM_PORT	0x060000

#define SYMBOL_TABLE_SIZE 29989		/* should always be prime */
static symbol_table symtab;

#define AUTH_TABLE_SIZE 397		/* should always be prime */
static symbol_table authtab = NULL;
static symbol_table channeltab = NULL;

static zone_config current_zone;
static int should_install;

static options current_options;
static int seen_options;
static int logged_options_error;

static controls current_controls;

static int seen_topology;

static server_config current_server;
static int seen_server;

static char *current_algorithm;
static char *current_secret;

static log_config current_logging;
static int current_category;
static int chan_type;
static int chan_level;
static u_int chan_flags;
static int chan_facility;
static char *chan_name;
static int chan_versions;
static u_long chan_max_size;

static log_channel lookup_channel(char *);
static void define_channel(const char *, log_channel);
static char *canonical_name(char *);

int yyparse();
	
#line 106 "ns_parser.y"
typedef union {
	char *			cp;
	int			s_int;
	long			num;
	u_long			ul_int;
	u_int16_t		us_int;
	union ns_sockaddr	ip_addr;
	ip_match_element	ime;
	ip_match_list		iml;
	rrset_order_list	rol;
	rrset_order_element	roe;
	struct dst_key *	keyi;
	enum axfr_format	axfr_fmt;
} YYSTYPE;
#line 139 "y.tab.c"
#define YYERRCODE 256
#define L_EOS 257
#define L_IPADDR 258
#define L_NUMBER 259
#define L_STRING 260
#define L_QSTRING 261
#define L_END_INCLUDE 262
#define T_INCLUDE 263
#define T_OPTIONS 264
#define T_DIRECTORY 265
#define T_PIDFILE 266
#define T_NAMED_XFER 267
#define T_DUMP_FILE 268
#define T_STATS_FILE 269
#define T_MEMSTATS_FILE 270
#define T_FAKE_IQUERY 271
#define T_RECURSION 272
#define T_FETCH_GLUE 273
#define T_HITCOUNT 274
#define T_PREFERRED_GLUE 275
#define T_QUERY_SOURCE 276
#define T_QUERY_SOURCE_V6 277
#define T_LISTEN_ON 278
#define T_LISTEN_ON_V6 279
#define T_PORT 280
#define T_ADDRESS 281
#define T_RRSET_ORDER 282
#define T_ORDER 283
#define T_NAME 284
#define T_CLASS 285
#define T_CONTROLS 286
#define T_INET 287
#define T_UNIX 288
#define T_PERM 289
#define T_OWNER 290
#define T_GROUP 291
#define T_ALLOW 292
#define T_DATASIZE 293
#define T_STACKSIZE 294
#define T_CORESIZE 295
#define T_DEFAULT 296
#define T_UNLIMITED 297
#define T_FILES 298
#define T_VERSION 299
#define T_HOSTNAME 300
#define T_HOSTSTATS 301
#define T_HOSTSTATSMAX 302
#define T_DEALLOC_ON_EXIT 303
#define T_TRANSFERS_IN 304
#define T_TRANSFERS_OUT 305
#define T_TRANSFERS_PER_NS 306
#define T_TRANSFER_FORMAT 307
#define T_MAX_TRANSFER_TIME_IN 308
#define T_SERIAL_QUERIES 309
#define T_ONE_ANSWER 310
#define T_MANY_ANSWERS 311
#define T_NOTIFY 312
#define T_EXPLICIT 313
#define T_NOTIFY_INITIAL 314
#define T_AUTH_NXDOMAIN 315
#define T_MULTIPLE_CNAMES 316
#define T_USE_IXFR 317
#define T_MAINTAIN_IXFR_BASE 318
#define T_CLEAN_INTERVAL 319
#define T_INTERFACE_INTERVAL 320
#define T_STATS_INTERVAL 321
#define T_MAX_LOG_SIZE_IXFR 322
#define T_HEARTBEAT 323
#define T_USE_ID_POOL 324
#define T_MAX_NCACHE_TTL 325
#define T_HAS_OLD_CLIENTS 326
#define T_RFC2308_TYPE1 327
#define T_LAME_TTL 328
#define T_MIN_ROOTS 329
#define T_TREAT_CR_AS_SPACE 330
#define T_EDNS_UDP_SIZE 331
#define T_LOGGING 332
#define T_CATEGORY 333
#define T_CHANNEL 334
#define T_SEVERITY 335
#define T_DYNAMIC 336
#define T_FILE 337
#define T_VERSIONS 338
#define T_SIZE 339
#define T_SYSLOG 340
#define T_DEBUG 341
#define T_NULL_OUTPUT 342
#define T_PRINT_TIME 343
#define T_PRINT_CATEGORY 344
#define T_PRINT_SEVERITY 345
#define T_SORTLIST 346
#define T_TOPOLOGY 347
#define T_SERVER 348
#define T_LONG_AXFR 349
#define T_BOGUS 350
#define T_TRANSFERS 351
#define T_KEYS 352
#define T_SUPPORT_IXFR 353
#define T_EDNS 354
#define T_ZONE 355
#define T_IN 356
#define T_CHAOS 357
#define T_HESIOD 358
#define T_TYPE 359
#define T_MASTER 360
#define T_SLAVE 361
#define T_STUB 362
#define T_RESPONSE 363
#define T_HINT 364
#define T_MASTERS 365
#define T_TRANSFER_SOURCE 366
#define T_TRANSFER_SOURCE_V6 367
#define T_PUBKEY 368
#define T_ALSO_NOTIFY 369
#define T_DIALUP 370
#define T_FILE_IXFR 371
#define T_IXFR_TMP 372
#define T_TRUSTED_KEYS 373
#define T_ACL 374
#define T_ALLOW_UPDATE 375
#define T_ALLOW_QUERY 376
#define T_ALLOW_TRANSFER 377
#define T_ALLOW_RECURSION 378
#define T_BLACKHOLE 379
#define T_SEC_KEY 380
#define T_ALGID 381
#define T_SECRET 382
#define T_CHECK_NAMES 383
#define T_WARN 384
#define T_FAIL 385
#define T_IGNORE 386
#define T_FORWARD 387
#define T_FORWARDERS 388
#define T_ONLY 389
#define T_FIRST 390
#define T_IF_NO_ANSWER 391
#define T_IF_NO_DOMAIN 392
#define T_YES 393
#define T_TRUE 394
#define T_NO 395
#define T_FALSE 396
const short yylhs[] = {                                        -1,
    0,   31,   31,   32,   32,   32,   32,   32,   32,   32,
   32,   32,   32,   32,   32,   33,   42,   34,   43,   43,
   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
   46,   44,   44,   44,   44,   44,   44,   44,   44,   44,
   50,   44,   44,   44,   44,   44,   44,   44,   44,   44,
   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
   44,   44,   44,   35,   54,   54,   55,   55,   55,   55,
   55,   55,   15,   15,   12,   12,   13,   13,   14,   14,
   16,    6,    6,    5,    5,    4,    4,   57,   58,   48,
   48,   48,   48,   59,   60,   49,   49,   49,   49,    2,
    2,    3,    3,   29,   29,   29,   29,   29,   27,   27,
   27,   28,   28,   28,   45,   45,   45,   45,   52,   52,
   52,   52,   26,   26,   26,   26,   53,   53,   53,   47,
   47,   61,   61,   62,   51,   51,   63,   63,   64,   65,
   36,   66,   66,   66,   68,   67,   70,   67,   72,   72,
   72,   72,   73,   73,   74,   75,   75,   75,   75,   75,
   76,   10,   10,   11,   11,   77,   78,   78,   78,   78,
   78,   78,   78,   71,   71,   71,    9,    9,   79,   69,
   69,   69,    8,    8,    8,    7,   80,   37,   81,   81,
   82,   82,   82,   82,   82,   82,   82,   20,   20,   18,
   18,   18,   17,   17,   17,   17,   17,   19,   23,   84,
   83,   83,   83,   85,   56,   56,   56,   86,   41,   87,
   87,   87,   24,   25,   40,   89,   38,   88,   88,   21,
   21,   22,   22,   22,   22,   22,   90,   90,   91,   91,
   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,
   94,   91,   91,   91,   91,   91,   91,   91,   91,   91,
   91,   91,   92,   92,   97,   97,   96,   96,   98,   98,
   99,   93,   93,   95,   95,  100,  100,  101,   39,  102,
  102,  103,  103,    1,   30,   30,
};
const short yylen[] = {                                         2,
    1,    1,    2,    1,    2,    2,    2,    2,    2,    2,
    2,    2,    1,    2,    2,    3,    0,    5,    2,    3,
    0,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    3,    2,    2,    5,    5,    2,
    0,    5,    2,    2,    2,    2,    4,    4,    4,    4,
    0,    5,    4,    4,    1,    1,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    4,    2,
    2,    2,    1,    4,    2,    3,    0,    8,   10,   12,
    8,    1,    2,    3,    0,    2,    0,    2,    0,    2,
    5,    1,    1,    1,    1,    1,    1,    2,    2,    1,
    1,    2,    2,    2,    2,    1,    1,    2,    2,    0,
    2,    0,    2,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    2,    2,
    2,    2,    1,    1,    1,    1,    2,    2,    2,    0,
    1,    2,    3,    1,    0,    1,    2,    3,    1,    0,
    5,    2,    3,    1,    0,    6,    0,    6,    1,    1,
    2,    1,    2,    2,    2,    0,    1,    1,    2,    2,
    3,    1,    1,    0,    1,    2,    1,    1,    1,    2,
    2,    2,    2,    2,    3,    1,    1,    1,    1,    2,
    3,    1,    1,    1,    1,    1,    0,    6,    2,    3,
    2,    2,    2,    2,    4,    2,    1,    2,    3,    1,
    2,    2,    1,    3,    3,    1,    3,    1,    1,    1,
    2,    3,    1,    1,    2,    3,    1,    0,    6,    2,
    2,    1,    3,    3,    5,    0,    5,    0,    3,    0,
    1,    1,    1,    1,    1,    1,    2,    3,    2,    2,
    2,    2,    5,    2,    2,    2,    4,    4,    4,    2,
    0,    5,    2,    2,    2,    2,    2,    5,    5,    4,
    2,    1,    2,    3,    1,    3,    0,    1,    2,    3,
    1,    1,    1,    0,    1,    2,    3,    1,    4,    2,
    3,    5,    5,    1,    1,    1,
};
const short yydefred[] = {                                      0,
    0,   13,    0,   17,    0,  160,    0,    0,    0,    0,
  238,    0,    0,    2,    4,    0,    0,    0,    0,    0,
    0,    0,    0,   14,   15,    0,    0,    0,    0,  207,
    0,    0,  305,  306,    0,    0,    3,    5,    6,    7,
    8,    9,   10,   11,   12,   16,    0,   92,    0,    0,
    0,    0,    0,    0,  246,  251,    0,    0,    0,    0,
    0,   83,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   61,    0,    0,    0,
    0,    0,    0,    0,   51,    0,    0,   65,   66,  104,
  105,    0,    0,   84,    0,   85,  164,    0,    0,    0,
    0,    0,    0,    0,    0,  299,    0,  300,    0,    0,
    0,    0,    0,  220,    0,  226,    0,  228,    0,   24,
   26,   25,   29,   27,   28,  128,  124,  125,  126,  127,
   31,   32,   33,   34,   30,    0,    0,   53,    0,    0,
    0,    0,   54,    0,    0,    0,    0,    0,    0,  144,
  145,  146,  139,  143,  140,  141,  142,   23,   22,   38,
   73,   39,  147,  148,  149,  102,  103,   67,   68,   69,
   35,   36,   37,   43,   44,   40,   41,   70,   71,   72,
   74,   77,   46,   75,   42,   47,   76,   81,   80,   82,
    0,    0,   55,   56,    0,   78,    0,    0,    0,    0,
  129,  130,  131,    0,  135,  136,  137,  138,   50,    0,
   18,    0,   19,    0,    0,    0,   86,  204,  205,  165,
  206,  203,  198,  167,  197,  161,    0,  162,  217,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  247,    0,
    0,  301,    0,    0,  222,    0,  221,  218,  245,    0,
  242,    0,    0,    0,    0,    0,  304,  107,  106,  109,
  108,  112,  113,  115,  114,  118,  119,  121,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  132,  133,  134,   45,    0,   20,    0,    0,    0,
    0,    0,  163,  214,  211,  213,    0,  212,  216,  208,
    0,  209,  282,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  271,    0,    0,    0,    0,  224,  225,  227,
  219,    0,    0,  240,  241,  239,    0,    0,   96,    0,
    0,   79,    0,   93,   60,   64,  159,    0,    0,    0,
   57,   59,   58,   63,  154,    0,    0,    0,    0,    0,
    0,    0,    0,  233,  230,  229,    0,    0,  210,  273,
  275,  276,  277,  274,  260,  252,  253,  255,  254,  256,
  259,    0,    0,  264,  265,    0,    0,    0,  281,  261,
  262,    0,    0,    0,  266,  292,  293,  270,    0,  249,
    0,  257,  302,  303,  243,  244,   48,   49,   98,    0,
    0,   94,   62,    0,  157,   52,    0,  152,    0,    0,
    0,  202,  199,    0,    0,  196,    0,    0,    0,  189,
    0,    0,    0,    0,  187,  188,    0,  215,    0,  231,
  123,    0,    0,    0,  291,    0,    0,    0,    0,    0,
    0,    0,  258,  100,    0,  158,  153,    0,    0,    0,
  166,    0,  200,  172,    0,  169,  190,    0,  183,  185,
  186,  182,  191,  192,  193,  168,    0,  194,  232,    0,
    0,    0,    0,    0,  280,    0,  289,  267,  268,  269,
  298,    0,    0,    0,  101,    0,    0,   91,  201,  171,
    0,    0,    0,    0,  181,  195,    0,  263,    0,  283,
  278,  279,  290,  272,    0,  296,    0,  237,  234,    0,
    0,  173,  174,  175,  179,  180,  286,  284,  297,    0,
   89,    0,  235,    0,  236,   90,
};
const short yydgoto[] = {                                      12,
  299,  187,  423,  300,  132,  208,  260,  261,  463,  510,
  511,  312,  381,  451,  313,  314,  154,  155,  156,  157,
   55,  421,  559,  294,  295,  193,  244,  325,  171,  158,
   13,   14,   15,   16,   17,   18,   19,   20,   21,   22,
   23,   27,  126,  127,  249,  250,  396,  178,  183,  235,
  388,  128,  129,   51,   52,  560,  179,  180,  184,  185,
  397,  398,  389,  390,   29,  140,  141,  331,  464,  332,
  474,  507,  543,  544,  545,  475,  476,  477,  465,   54,
  276,  277,  407,  408,  561,   36,  296,  279,  143,  364,
  365,  521,  438,  439,  532,  486,  522,  487,  488,  533,
  534,   58,   59,
};
const short yysindex[] = {                                    155,
 -195,    0, -243,    0,  -82,    0, -206, -196,  -50, -203,
    0,    0,  155,    0,    0, -175, -173, -160, -158, -156,
 -150, -143, -141,    0,    0, -127,  -11, -168,    9,    0,
 -203, -125,    0,    0,   18, -203,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  460,    0,   -4, -101,
  -97,  -92, -145,   45,    0,    0, -184,  -88,  -87,   80,
   57,    0,  -75,  -69,  -67,  -63,  -61,  -53, -232, -232,
 -232, -232,  -55, -190, -157,  -59,  -59,  101,   65,   65,
   65,   65,  -16,   -6, -232,    8, -232,   40,   73,   75,
 -162,   83,   89, -180, -232, -232, -232, -232, -232,   91,
   99,  105,   65,  106, -232,  107, -232, -232,  108,  109,
 -232,  110,  154,  237,   -4,   -4,    0, -232,  248,  250,
  251,  254, -159, -172,    0,  305,  121,    0,    0,    0,
    0, -200,   90,    0,  123,    0,    0,   37, -211,  -86,
  128,  119,  267,  134,  139,    0,  143,    0,  358,  362,
  150,   80, -108,    0,  157,    0,  -29,    0, -239,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  -31,   -4,    0,  132,  135,
  -31,   -4,    0,  140,  146,  156,  306,  308,  149,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   80,   80,    0,    0,  315,    0,   80,   80,   80,   80,
    0,    0,    0, -120,    0,    0,    0,    0,    0,  316,
    0,  183,    0,  156,  321,  190,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  194,    0,    0, -162,
 -232,  193,  330, -232, -232,   -7,  198,  125,    0,  197,
  199,    0,  205,  206,    0,  -25,    0,    0,    0,  209,
    0, -203, -203,   85,   93,  350,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   80,   80,
 -203,  117, -111,  220,  -21,  -17,  224,   -8,    2,   11,
   15,    0,    0,    0,    0,  225,    0,  186,   80,  195,
  363,  365,    0,    0,    0,    0, -151,    0,    0,    0,
  232,    0,    0,  239, -137, -232,   65,  238,   61,  227,
   -4,   -4,   29,  381, -232,  244,  256,  386,  388,  392,
 -120,  -71,    0,  336,  261,  259,  264,    0,    0,    0,
    0,  269,  273,    0,    0,    0,   33,   44,    0, -203,
  247,    0,  276,    0,    0,    0,    0,  409,  224,  279,
    0,    0,    0,    0,    0,  413,  225,  282,  417,   48,
  283, -205,  179,    0,    0,    0, -106,  284,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  156,  420,    0,    0,  285,  286,  288,    0,    0,
    0,   80,   80,   80,    0,    0,    0,    0,  424,    0,
  291,    0,    0,    0,    0,    0,    0,    0,    0,  290,
  266,    0,    0,  295,    0,    0,  296,    0,   80,  202,
  265,    0,    0, -116,  298,    0, -114,  301, -174,    0,
 -232, -232, -232,   59,    0,    0,  310,    0,  311,    0,
    0,  307,  304,  326,    0,  444,  288,  329,   52,   56,
   62,  331,    0,    0,  328,    0,    0,   70,  467,  332,
    0,  337,    0,    0,  334,    0,    0,   -3,    0,    0,
    0,    0,    0,    0,    0,    0,  338,    0,    0,  216,
  -80,  340,  341,  354,    0,  344,    0,    0,    0,    0,
    0,  491,  331,  361,    0,  287,  -49,    0,    0,    0,
 -223,   65,  299,  302,    0,    0, -203,    0,  380,    0,
    0,    0,    0,    0,  384,    0,  519,    0,    0,  -78,
  389,    0,    0,    0,    0,    0,    0,    0,    0,  -49,
    0,  390,    0,  -64,    0,    0,
};
const short yyrindex[] = {                                      0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  643,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  393,    0,    0,
 -100,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  396,    0,    0,    0,
  393,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  522,  522,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  396,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  398,    0,    0,    0,    0,    0,  399,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  400,  402,
    0,    0,    0,  403,  404,    0,    0,    0, -220,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   69, -220,    0,    0,    0,  524,    0,    0,    0,
    0,    0,    0,    0,    0,  537,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  540,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  382,    0,    0,    0,    0,    0,    0,    0,  539,    0,
    0,    0,    0,    0,    0,    0,  541,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  542,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  411,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  544,    0,    0,    0,
    0,  545,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  419,    0,    0,  421,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  422,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  552,    0,    0,  423,    0,    0,    0,    0,
    0,    0,  428,  429,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,
};
const short yygindex[] = {                                      0,
 -176,  610,    0,  508,  -56,  426,    0,    0,  551,    0,
    0,    0,    0,    0,    0,  378,  546,   68,    0,   47,
    0,    0, -324,  405,  415,  -79,    0,  333,  -65,  -10,
    0,  684,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  572,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  659,  144,  535,  538,  533,  536,
    0,  324,    0,  351,    0,    0,  582,    0,    0,    0,
    0,    0,  200,  203,    0,    0,    0,  271,  277,    0,
    0,  471,    0,  342, -506,    0,    0,    0,    0,    0,
  379,    0,    0,    0,    0,    0,  229,    0,  270,    0,
  215,    0,  693,
};
#define YYTABLESIZE 848
const short yytable[] = {                                      35,
  195,  196,  197,  153,  172,  173,  174,  153,  501,  308,
  298,  153,  405,  382,  152,  153,  291,   26,  478,  200,
   56,  202,  250,  221,  153,   61,  166,  134,  212,  213,
  214,  215,  216,  217,  153,  562,  146,  131,  266,  223,
   28,  225,  226,  153,  548,  229,  571,  153,   33,   34,
  462,   30,  236,  572,   33,   34,   33,   34,  233,  234,
  576,   24,   95,   95,   31,  153,   25,  572,  194,  194,
  194,  194,   32,  563,  144,  145,  153,  328,  166,  254,
  153,   38,  405,   39,  153,   33,   34,   48,  153,  176,
  177,  255,  194,  152,  153,  289,   40,  152,   41,  370,
   42,  152,  153,  385,  404,  152,   43,  386,   33,   34,
  137,   47,  153,   44,  152,   45,  391,  340,   49,   50,
  301,  166,  181,  182,  152,  305,  392,  262,  265,   46,
  263,   53,  211,  152,   57,  393,  263,  152,   95,  394,
   60,  292,  293,   33,   34,   33,   34,  206,  207,  149,
  150,   33,   34,   33,   34,  152,  250,  447,   48,  133,
  167,  168,  169,  170,  136,  509,  152,  142,  448,  148,
  152,   57,  460,  311,  152,  411,  528,  520,  152,  159,
  529,   33,   34,  516,  152,  160,  530,  138,  139,   49,
   50,  161,  152,  162,  536,   33,   34,  163,  286,  164,
  241,  242,  152,  243,  175,  335,  558,  165,  338,  339,
   33,   34,  167,  168,  169,  170,  245,  246,  247,  248,
  186,  504,  567,  189,  290,  263,  505,  297,  149,  150,
   33,   34,  149,  150,   33,   34,  149,  150,   33,   34,
  149,  150,   33,   34,  198,  481,  138,  139,  269,  149,
  150,   33,   34,  130,  199,  167,  168,  169,  170,  149,
  150,   33,   34,  322,  323,  324,  201,  414,  149,  150,
   33,   34,  149,  150,   33,   34,  231,  315,  316,  412,
  413,  372,  373,  318,  319,  320,  321,  426,  427,  429,
  149,  150,   33,   34,  424,  425,   33,   34,  203,  270,
  379,  149,  150,   33,   34,  149,  150,   33,   34,  149,
  150,   33,   34,  149,  150,   33,   34,  436,  437,  149,
  150,   33,   34,  190,   33,   34,  406,  149,  150,   33,
   34,  204,  258,  205,  541,  542,  194,  149,  150,   33,
   34,  209,  271,  272,  273,  274,  275,  210,  259,  218,
  151,   97,   97,  290,  151,  377,  378,  219,  151,  232,
  191,  192,  151,  220,  222,  224,  227,  228,  230,  449,
  237,  151,  238,  239,  269,  400,  240,  253,  256,  257,
  343,  151,  290,  290,  268,  290,  290,  290,  290,  278,
  151,  265,  280,  467,  151,  468,  406,  281,  469,  282,
  470,  471,  472,  473,  283,  513,  514,  515,  284,  285,
    1,  176,  151,  288,  297,  177,    2,    3,    4,  181,
  416,  417,  418,  151,  419,  270,  182,  151,  309,  251,
  310,  151,  344,  311,  466,  151,  345,  317,  326,  327,
    5,  151,  346,  329,  290,  290,  347,  420,  330,  151,
  333,  336,  337,  265,  342,  366,  506,  367,  512,  151,
  440,  348,  564,  368,  369,  371,  293,  290,  271,  272,
  273,  274,  275,  292,  376,  380,  384,  399,  489,  490,
  491,  387,  395,  349,  401,  402,    6,  403,  409,  350,
  351,  352,  353,  354,  355,  356,  357,  410,  415,  358,
  359,  360,    7,  428,  430,  498,  422,  361,  432,    8,
  433,  362,  363,  467,  434,  468,  431,  442,  469,  443,
  470,  471,  472,  473,  444,  445,  406,    9,   10,  446,
  450,  194,  452,  453,   11,  455,  406,  456,  458,  459,
  480,  461,  482,  483,  484,  485,  492,  493,  495,  406,
  494,  496,  497,  499,  503,  500,  290,  290,  290,  406,
   62,  508,  523,  406,  520,  290,  518,  519,  525,   63,
   64,   65,   66,   67,   68,   69,   70,   71,   72,   73,
   74,   75,   76,   77,  524,  527,   78,  535,  531,  537,
  538,  343,  540,  539,  546,  547,  550,   79,   80,   81,
  553,  551,   82,   83,   84,   85,   86,   87,   88,   89,
   90,   91,   92,   93,  552,  554,   94,  556,   95,   96,
   97,   98,   99,  100,  101,  102,  103,  104,  105,  106,
  107,  108,  109,  110,  111,  112,  568,  542,  557,  541,
  569,  570,    1,  344,  120,  573,  575,  345,  155,   87,
  113,  114,   21,  346,  248,  223,  110,  347,  111,  116,
  117,  150,  122,  156,   99,  151,  287,  184,  288,  294,
  115,  116,  348,  117,  118,  170,  295,  176,  285,   88,
  119,  120,  121,  122,  177,  178,  188,  123,  304,  264,
  383,  124,  125,  435,  349,  334,   37,  252,  287,  375,
  350,  351,  352,  353,  354,  355,  356,  357,  374,  135,
  358,  359,  360,  574,  303,   62,  302,  307,  361,  306,
  457,  267,  362,  363,   63,   64,   65,   66,   67,   68,
   69,   70,   71,   72,   73,   74,   75,   76,   77,  454,
  502,   78,  441,  566,  517,  565,  341,  555,  479,  549,
  147,    0,   79,   80,   81,    0,  526,   82,   83,   84,
   85,   86,   87,   88,   89,   90,   91,   92,   93,    0,
    0,   94,    0,   95,   96,   97,   98,   99,  100,  101,
  102,  103,  104,  105,  106,  107,  108,  109,  110,  111,
  112,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  113,  114,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  115,  116,    0,  117,  118,
    0,    0,    0,    0,    0,  119,  120,  121,  122,    0,
    0,    0,  123,    0,    0,    0,  124,  125,
};
const short yycheck[] = {                                      10,
   80,   81,   82,   33,   70,   71,   72,   33,  125,  186,
   42,   33,  337,  125,  123,   33,  256,  261,  125,   85,
   31,   87,  123,  103,   33,   36,  259,  125,   94,   95,
   96,   97,   98,   99,   33,  259,  125,   42,  125,  105,
  123,  107,  108,   33,  125,  111,  125,   33,  260,  261,
  256,  258,  118,  560,  260,  261,  260,  261,  115,  116,
  125,  257,  283,  284,  261,   33,  262,  574,   79,   80,
   81,   82,  123,  297,  259,  260,   33,  254,  259,  280,
   33,  257,  407,  257,   33,  260,  261,  256,   33,  280,
  281,  292,  103,  123,   33,  125,  257,  123,  257,  125,
  257,  123,   33,  125,  256,  123,  257,  125,  260,  261,
  256,  123,   33,  257,  123,  257,  125,  125,  287,  288,
  177,  259,  280,  281,  123,  182,  125,  138,  139,  257,
  342,  123,  313,  123,  260,  125,  342,  123,  359,  125,
  123,  381,  382,  260,  261,  260,  261,  310,  311,  258,
  259,  260,  261,  260,  261,  123,  257,  125,  256,  261,
  393,  394,  395,  396,  257,  340,  123,  123,  125,  257,
  123,  260,  125,  285,  123,  313,  125,  258,  123,  123,
  125,  260,  261,  125,  123,  261,  125,  333,  334,  287,
  288,  261,  123,  261,  125,  260,  261,  261,  152,  261,
  360,  361,  123,  363,  260,  271,  256,  261,  274,  275,
  260,  261,  393,  394,  395,  396,  389,  390,  391,  392,
  280,  336,  547,  123,  157,  342,  341,  259,  258,  259,
  260,  261,  258,  259,  260,  261,  258,  259,  260,  261,
  258,  259,  260,  261,  261,  422,  333,  334,  256,  258,
  259,  260,  261,  258,  261,  393,  394,  395,  396,  258,
  259,  260,  261,  384,  385,  386,  259,  347,  258,  259,
  260,  261,  258,  259,  260,  261,  123,  231,  232,  345,
  346,  292,  293,  237,  238,  239,  240,  259,  260,  355,
  258,  259,  260,  261,  351,  352,  260,  261,  259,  307,
  311,  258,  259,  260,  261,  258,  259,  260,  261,  258,
  259,  260,  261,  258,  259,  260,  261,  389,  390,  258,
  259,  260,  261,  259,  260,  261,  337,  258,  259,  260,
  261,  259,  296,  259,  338,  339,  347,  258,  259,  260,
  261,  259,  350,  351,  352,  353,  354,  259,  312,  259,
  380,  283,  284,  286,  380,  309,  310,  259,  380,  123,
  296,  297,  380,  259,  259,  259,  259,  259,  259,  380,
  123,  380,  123,  123,  256,  329,  123,  257,  289,  257,
  256,  380,  315,  316,  257,  318,  319,  320,  321,  123,
  380,  402,  259,  335,  380,  337,  407,  259,  340,  257,
  342,  343,  344,  345,   47,  471,  472,  473,   47,  260,
  256,  280,  380,  257,  259,  281,  262,  263,  264,  280,
  360,  361,  362,  380,  364,  307,  281,  380,  123,  125,
  123,  380,  308,  285,  256,  380,  312,  123,  123,  257,
  286,  380,  318,  123,  377,  378,  322,  387,  259,  380,
  257,  259,  123,  464,  257,  259,  467,  259,  469,  380,
  125,  337,  542,  259,  259,  257,  382,  400,  350,  351,
  352,  353,  354,  381,  125,  359,  257,  292,  432,  433,
  434,  258,  258,  359,  290,  123,  332,  123,  257,  365,
  366,  367,  368,  369,  370,  371,  372,  259,  261,  375,
  376,  377,  348,  123,  261,  459,  280,  383,  123,  355,
  123,  387,  388,  335,  123,  337,  261,  257,  340,  261,
  342,  343,  344,  345,  261,  257,  537,  373,  374,  257,
  284,  542,  257,  125,  380,  257,  547,  125,  257,  123,
  257,  259,  123,  259,  259,  258,  123,  257,  283,  560,
  261,  257,  257,  352,  257,  291,  489,  490,  491,  570,
  256,  261,  259,  574,  258,  498,  257,  257,  125,  265,
  266,  267,  268,  269,  270,  271,  272,  273,  274,  275,
  276,  277,  278,  279,  259,  257,  282,  260,  258,  123,
  259,  256,  259,  257,  257,  380,  257,  293,  294,  295,
  257,  261,  298,  299,  300,  301,  302,  303,  304,  305,
  306,  307,  308,  309,  261,  125,  312,  257,  314,  315,
  316,  317,  318,  319,  320,  321,  322,  323,  324,  325,
  326,  327,  328,  329,  330,  331,  257,  339,  352,  338,
  257,  123,    0,  308,  123,  257,  257,  312,  125,  257,
  346,  347,  257,  318,  257,  257,  257,  322,  257,  257,
  257,  125,  123,  125,  283,  125,  125,  257,  125,  125,
  366,  367,  337,  369,  370,  257,  125,  257,  257,  257,
  376,  377,  378,  379,  257,  257,   77,  383,  181,  139,
  313,  387,  388,  361,  359,  270,   13,  126,  153,  295,
  365,  366,  367,  368,  369,  370,  371,  372,  294,   51,
  375,  376,  377,  570,  180,  256,  179,  185,  383,  184,
  397,  140,  387,  388,  265,  266,  267,  268,  269,  270,
  271,  272,  273,  274,  275,  276,  277,  278,  279,  389,
  464,  282,  364,  544,  474,  543,  276,  533,  407,  521,
   58,   -1,  293,  294,  295,   -1,  487,  298,  299,  300,
  301,  302,  303,  304,  305,  306,  307,  308,  309,   -1,
   -1,  312,   -1,  314,  315,  316,  317,  318,  319,  320,
  321,  322,  323,  324,  325,  326,  327,  328,  329,  330,
  331,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  346,  347,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  366,  367,   -1,  369,  370,
   -1,   -1,   -1,   -1,   -1,  376,  377,  378,  379,   -1,
   -1,   -1,  383,   -1,   -1,   -1,  387,  388,
};
#define YYFINAL 12
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 396
#if YYDEBUG
const char * const yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
"'!'",0,0,0,0,0,0,0,0,"'*'",0,0,0,0,"'/'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"'{'",0,"'}'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"L_EOS",
"L_IPADDR","L_NUMBER","L_STRING","L_QSTRING","L_END_INCLUDE","T_INCLUDE",
"T_OPTIONS","T_DIRECTORY","T_PIDFILE","T_NAMED_XFER","T_DUMP_FILE",
"T_STATS_FILE","T_MEMSTATS_FILE","T_FAKE_IQUERY","T_RECURSION","T_FETCH_GLUE",
"T_HITCOUNT","T_PREFERRED_GLUE","T_QUERY_SOURCE","T_QUERY_SOURCE_V6",
"T_LISTEN_ON","T_LISTEN_ON_V6","T_PORT","T_ADDRESS","T_RRSET_ORDER","T_ORDER",
"T_NAME","T_CLASS","T_CONTROLS","T_INET","T_UNIX","T_PERM","T_OWNER","T_GROUP",
"T_ALLOW","T_DATASIZE","T_STACKSIZE","T_CORESIZE","T_DEFAULT","T_UNLIMITED",
"T_FILES","T_VERSION","T_HOSTNAME","T_HOSTSTATS","T_HOSTSTATSMAX",
"T_DEALLOC_ON_EXIT","T_TRANSFERS_IN","T_TRANSFERS_OUT","T_TRANSFERS_PER_NS",
"T_TRANSFER_FORMAT","T_MAX_TRANSFER_TIME_IN","T_SERIAL_QUERIES","T_ONE_ANSWER",
"T_MANY_ANSWERS","T_NOTIFY","T_EXPLICIT","T_NOTIFY_INITIAL","T_AUTH_NXDOMAIN",
"T_MULTIPLE_CNAMES","T_USE_IXFR","T_MAINTAIN_IXFR_BASE","T_CLEAN_INTERVAL",
"T_INTERFACE_INTERVAL","T_STATS_INTERVAL","T_MAX_LOG_SIZE_IXFR","T_HEARTBEAT",
"T_USE_ID_POOL","T_MAX_NCACHE_TTL","T_HAS_OLD_CLIENTS","T_RFC2308_TYPE1",
"T_LAME_TTL","T_MIN_ROOTS","T_TREAT_CR_AS_SPACE","T_EDNS_UDP_SIZE","T_LOGGING",
"T_CATEGORY","T_CHANNEL","T_SEVERITY","T_DYNAMIC","T_FILE","T_VERSIONS",
"T_SIZE","T_SYSLOG","T_DEBUG","T_NULL_OUTPUT","T_PRINT_TIME","T_PRINT_CATEGORY",
"T_PRINT_SEVERITY","T_SORTLIST","T_TOPOLOGY","T_SERVER","T_LONG_AXFR","T_BOGUS",
"T_TRANSFERS","T_KEYS","T_SUPPORT_IXFR","T_EDNS","T_ZONE","T_IN","T_CHAOS",
"T_HESIOD","T_TYPE","T_MASTER","T_SLAVE","T_STUB","T_RESPONSE","T_HINT",
"T_MASTERS","T_TRANSFER_SOURCE","T_TRANSFER_SOURCE_V6","T_PUBKEY",
"T_ALSO_NOTIFY","T_DIALUP","T_FILE_IXFR","T_IXFR_TMP","T_TRUSTED_KEYS","T_ACL",
"T_ALLOW_UPDATE","T_ALLOW_QUERY","T_ALLOW_TRANSFER","T_ALLOW_RECURSION",
"T_BLACKHOLE","T_SEC_KEY","T_ALGID","T_SECRET","T_CHECK_NAMES","T_WARN",
"T_FAIL","T_IGNORE","T_FORWARD","T_FORWARDERS","T_ONLY","T_FIRST",
"T_IF_NO_ANSWER","T_IF_NO_DOMAIN","T_YES","T_TRUE","T_NO","T_FALSE",
};
const char * const yyrule[] = {
"$accept : config_file",
"config_file : statement_list",
"statement_list : statement",
"statement_list : statement_list statement",
"statement : include_stmt",
"statement : options_stmt L_EOS",
"statement : controls_stmt L_EOS",
"statement : logging_stmt L_EOS",
"statement : server_stmt L_EOS",
"statement : zone_stmt L_EOS",
"statement : trusted_keys_stmt L_EOS",
"statement : acl_stmt L_EOS",
"statement : key_stmt L_EOS",
"statement : L_END_INCLUDE",
"statement : error L_EOS",
"statement : error L_END_INCLUDE",
"include_stmt : T_INCLUDE L_QSTRING L_EOS",
"$$1 :",
"options_stmt : T_OPTIONS $$1 '{' options '}'",
"options : option L_EOS",
"options : options option L_EOS",
"option :",
"option : T_HOSTNAME L_QSTRING",
"option : T_VERSION L_QSTRING",
"option : T_DIRECTORY L_QSTRING",
"option : T_NAMED_XFER L_QSTRING",
"option : T_PIDFILE L_QSTRING",
"option : T_STATS_FILE L_QSTRING",
"option : T_MEMSTATS_FILE L_QSTRING",
"option : T_DUMP_FILE L_QSTRING",
"option : T_PREFERRED_GLUE L_STRING",
"option : T_FAKE_IQUERY yea_or_nay",
"option : T_RECURSION yea_or_nay",
"option : T_FETCH_GLUE yea_or_nay",
"option : T_HITCOUNT yea_or_nay",
"option : T_NOTIFY T_EXPLICIT",
"option : T_NOTIFY yea_or_nay",
"option : T_NOTIFY_INITIAL yea_or_nay",
"option : T_HOSTSTATS yea_or_nay",
"option : T_DEALLOC_ON_EXIT yea_or_nay",
"option : T_USE_IXFR yea_or_nay",
"option : T_MAINTAIN_IXFR_BASE yea_or_nay",
"option : T_HAS_OLD_CLIENTS yea_or_nay",
"option : T_AUTH_NXDOMAIN yea_or_nay",
"option : T_MULTIPLE_CNAMES yea_or_nay",
"option : T_CHECK_NAMES check_names_type check_names_opt",
"option : T_USE_ID_POOL yea_or_nay",
"option : T_RFC2308_TYPE1 yea_or_nay",
"option : T_LISTEN_ON maybe_port '{' address_match_list '}'",
"option : T_LISTEN_ON_V6 maybe_port '{' address_match_list '}'",
"option : T_FORWARD forward_opt",
"$$2 :",
"option : T_FORWARDERS $$2 '{' opt_forwarders_list '}'",
"option : T_QUERY_SOURCE query_source",
"option : T_QUERY_SOURCE_V6 query_source_v6",
"option : T_TRANSFER_SOURCE maybe_wild_addr",
"option : T_TRANSFER_SOURCE_V6 maybe_wild_addr",
"option : T_ALLOW_QUERY '{' address_match_list '}'",
"option : T_ALLOW_RECURSION '{' address_match_list '}'",
"option : T_ALLOW_TRANSFER '{' address_match_list '}'",
"option : T_SORTLIST '{' address_match_list '}'",
"$$3 :",
"option : T_ALSO_NOTIFY $$3 '{' opt_also_notify_list '}'",
"option : T_BLACKHOLE '{' address_match_list '}'",
"option : T_TOPOLOGY '{' address_match_list '}'",
"option : size_clause",
"option : transfer_clause",
"option : T_TRANSFER_FORMAT transfer_format",
"option : T_MAX_TRANSFER_TIME_IN L_NUMBER",
"option : T_SERIAL_QUERIES L_NUMBER",
"option : T_CLEAN_INTERVAL L_NUMBER",
"option : T_INTERFACE_INTERVAL L_NUMBER",
"option : T_STATS_INTERVAL L_NUMBER",
"option : T_HOSTSTATSMAX L_NUMBER",
"option : T_MAX_LOG_SIZE_IXFR size_spec",
"option : T_MAX_NCACHE_TTL L_NUMBER",
"option : T_LAME_TTL L_NUMBER",
"option : T_HEARTBEAT L_NUMBER",
"option : T_DIALUP yea_or_nay",
"option : T_RRSET_ORDER '{' rrset_ordering_list '}'",
"option : T_TREAT_CR_AS_SPACE yea_or_nay",
"option : T_MIN_ROOTS L_NUMBER",
"option : T_EDNS_UDP_SIZE L_NUMBER",
"option : error",
"controls_stmt : T_CONTROLS '{' controls '}'",
"controls : control L_EOS",
"controls : controls control L_EOS",
"control :",
"control : T_INET maybe_wild_addr T_PORT in_port T_ALLOW '{' address_match_list '}'",
"control : T_INET maybe_wild_addr T_ALLOW '{' address_match_list '}' T_KEYS '{' dummy_key_list '}'",
"control : T_INET maybe_wild_addr T_PORT in_port T_ALLOW '{' address_match_list '}' T_KEYS '{' dummy_key_list '}'",
"control : T_UNIX L_QSTRING T_PERM L_NUMBER T_OWNER L_NUMBER T_GROUP L_NUMBER",
"control : error",
"rrset_ordering_list : rrset_ordering_element L_EOS",
"rrset_ordering_list : rrset_ordering_list rrset_ordering_element L_EOS",
"ordering_class :",
"ordering_class : T_CLASS any_string",
"ordering_type :",
"ordering_type : T_TYPE any_string",
"ordering_name :",
"ordering_name : T_NAME L_QSTRING",
"rrset_ordering_element : ordering_class ordering_type ordering_name T_ORDER L_STRING",
"transfer_format : T_ONE_ANSWER",
"transfer_format : T_MANY_ANSWERS",
"maybe_wild_addr : L_IPADDR",
"maybe_wild_addr : '*'",
"maybe_wild_port : in_port",
"maybe_wild_port : '*'",
"query_source_address : T_ADDRESS maybe_wild_addr",
"query_source_port : T_PORT maybe_wild_port",
"query_source : query_source_address",
"query_source : query_source_port",
"query_source : query_source_address query_source_port",
"query_source : query_source_port query_source_address",
"query_source_address_v6 : T_ADDRESS maybe_wild_addr",
"query_source_port_v6 : T_PORT maybe_wild_port",
"query_source_v6 : query_source_address_v6",
"query_source_v6 : query_source_port_v6",
"query_source_v6 : query_source_address_v6 query_source_port_v6",
"query_source_v6 : query_source_port_v6 query_source_address_v6",
"maybe_port :",
"maybe_port : T_PORT in_port",
"maybe_zero_port :",
"maybe_zero_port : T_PORT in_port",
"yea_or_nay : T_YES",
"yea_or_nay : T_TRUE",
"yea_or_nay : T_NO",
"yea_or_nay : T_FALSE",
"yea_or_nay : L_NUMBER",
"check_names_type : T_MASTER",
"check_names_type : T_SLAVE",
"check_names_type : T_RESPONSE",
"check_names_opt : T_WARN",
"check_names_opt : T_FAIL",
"check_names_opt : T_IGNORE",
"forward_opt : T_ONLY",
"forward_opt : T_FIRST",
"forward_opt : T_IF_NO_ANSWER",
"forward_opt : T_IF_NO_DOMAIN",
"size_clause : T_DATASIZE size_spec",
"size_clause : T_STACKSIZE size_spec",
"size_clause : T_CORESIZE size_spec",
"size_clause : T_FILES size_spec",
"size_spec : any_string",
"size_spec : L_NUMBER",
"size_spec : T_DEFAULT",
"size_spec : T_UNLIMITED",
"transfer_clause : T_TRANSFERS_IN L_NUMBER",
"transfer_clause : T_TRANSFERS_OUT L_NUMBER",
"transfer_clause : T_TRANSFERS_PER_NS L_NUMBER",
"opt_forwarders_list :",
"opt_forwarders_list : forwarders_in_addr_list",
"forwarders_in_addr_list : forwarders_in_addr L_EOS",
"forwarders_in_addr_list : forwarders_in_addr_list forwarders_in_addr L_EOS",
"forwarders_in_addr : L_IPADDR",
"opt_also_notify_list :",
"opt_also_notify_list : also_notify_in_addr_list",
"also_notify_in_addr_list : also_notify_in_addr L_EOS",
"also_notify_in_addr_list : also_notify_in_addr_list also_notify_in_addr L_EOS",
"also_notify_in_addr : L_IPADDR",
"$$4 :",
"logging_stmt : T_LOGGING $$4 '{' logging_opts_list '}'",
"logging_opts_list : logging_opt L_EOS",
"logging_opts_list : logging_opts_list logging_opt L_EOS",
"logging_opts_list : error",
"$$5 :",
"logging_opt : T_CATEGORY category $$5 '{' channel_list '}'",
"$$6 :",
"logging_opt : T_CHANNEL channel_name $$6 '{' channel_opt_list '}'",
"channel_severity : any_string",
"channel_severity : T_DEBUG",
"channel_severity : T_DEBUG L_NUMBER",
"channel_severity : T_DYNAMIC",
"version_modifier : T_VERSIONS L_NUMBER",
"version_modifier : T_VERSIONS T_UNLIMITED",
"size_modifier : T_SIZE size_spec",
"maybe_file_modifiers :",
"maybe_file_modifiers : version_modifier",
"maybe_file_modifiers : size_modifier",
"maybe_file_modifiers : version_modifier size_modifier",
"maybe_file_modifiers : size_modifier version_modifier",
"channel_file : T_FILE L_QSTRING maybe_file_modifiers",
"facility_name : any_string",
"facility_name : T_SYSLOG",
"maybe_syslog_facility :",
"maybe_syslog_facility : facility_name",
"channel_syslog : T_SYSLOG maybe_syslog_facility",
"channel_opt : channel_file",
"channel_opt : channel_syslog",
"channel_opt : T_NULL_OUTPUT",
"channel_opt : T_SEVERITY channel_severity",
"channel_opt : T_PRINT_TIME yea_or_nay",
"channel_opt : T_PRINT_CATEGORY yea_or_nay",
"channel_opt : T_PRINT_SEVERITY yea_or_nay",
"channel_opt_list : channel_opt L_EOS",
"channel_opt_list : channel_opt_list channel_opt L_EOS",
"channel_opt_list : error",
"channel_name : any_string",
"channel_name : T_NULL_OUTPUT",
"channel : channel_name",
"channel_list : channel L_EOS",
"channel_list : channel_list channel L_EOS",
"channel_list : error",
"category_name : any_string",
"category_name : T_DEFAULT",
"category_name : T_NOTIFY",
"category : category_name",
"$$7 :",
"server_stmt : T_SERVER L_IPADDR $$7 '{' server_info_list '}'",
"server_info_list : server_info L_EOS",
"server_info_list : server_info_list server_info L_EOS",
"server_info : T_BOGUS yea_or_nay",
"server_info : T_SUPPORT_IXFR yea_or_nay",
"server_info : T_TRANSFERS L_NUMBER",
"server_info : T_TRANSFER_FORMAT transfer_format",
"server_info : T_KEYS '{' key_list '}'",
"server_info : T_EDNS yea_or_nay",
"server_info : error",
"address_match_list : address_match_element L_EOS",
"address_match_list : address_match_list address_match_element L_EOS",
"address_match_element : address_match_simple",
"address_match_element : '!' address_match_simple",
"address_match_element : T_SEC_KEY L_STRING",
"address_match_simple : L_IPADDR",
"address_match_simple : L_IPADDR '/' L_NUMBER",
"address_match_simple : L_NUMBER '/' L_NUMBER",
"address_match_simple : address_name",
"address_match_simple : '{' address_match_list '}'",
"address_name : any_string",
"key_ref : any_string",
"key_list_element : key_ref",
"key_list : key_list_element L_EOS",
"key_list : key_list key_list_element L_EOS",
"key_list : error",
"dummy_key_list_element : key_ref",
"dummy_key_list : dummy_key_list_element L_EOS",
"dummy_key_list : dummy_key_list dummy_key_list_element L_EOS",
"dummy_key_list : error",
"$$8 :",
"key_stmt : T_SEC_KEY $$8 any_string '{' key_definition '}'",
"key_definition : algorithm_id secret",
"key_definition : secret algorithm_id",
"key_definition : error",
"algorithm_id : T_ALGID any_string L_EOS",
"secret : T_SECRET any_string L_EOS",
"acl_stmt : T_ACL any_string '{' address_match_list '}'",
"$$9 :",
"zone_stmt : T_ZONE L_QSTRING optional_class $$9 optional_zone_options_list",
"optional_zone_options_list :",
"optional_zone_options_list : '{' zone_option_list '}'",
"optional_class :",
"optional_class : any_string",
"zone_type : T_MASTER",
"zone_type : T_SLAVE",
"zone_type : T_HINT",
"zone_type : T_STUB",
"zone_type : T_FORWARD",
"zone_option_list : zone_option L_EOS",
"zone_option_list : zone_option_list zone_option L_EOS",
"zone_option : T_TYPE zone_type",
"zone_option : T_FILE L_QSTRING",
"zone_option : T_FILE_IXFR L_QSTRING",
"zone_option : T_IXFR_TMP L_QSTRING",
"zone_option : T_MASTERS maybe_zero_port '{' master_in_addr_list '}'",
"zone_option : T_TRANSFER_SOURCE maybe_wild_addr",
"zone_option : T_TRANSFER_SOURCE_V6 maybe_wild_addr",
"zone_option : T_CHECK_NAMES check_names_opt",
"zone_option : T_ALLOW_UPDATE '{' address_match_list '}'",
"zone_option : T_ALLOW_QUERY '{' address_match_list '}'",
"zone_option : T_ALLOW_TRANSFER '{' address_match_list '}'",
"zone_option : T_FORWARD zone_forward_opt",
"$$10 :",
"zone_option : T_FORWARDERS $$10 '{' opt_zone_forwarders_list '}'",
"zone_option : T_MAX_TRANSFER_TIME_IN L_NUMBER",
"zone_option : T_MAX_LOG_SIZE_IXFR size_spec",
"zone_option : T_NOTIFY T_EXPLICIT",
"zone_option : T_NOTIFY yea_or_nay",
"zone_option : T_MAINTAIN_IXFR_BASE yea_or_nay",
"zone_option : T_PUBKEY L_NUMBER L_NUMBER L_NUMBER L_QSTRING",
"zone_option : T_PUBKEY L_STRING L_NUMBER L_NUMBER L_QSTRING",
"zone_option : T_ALSO_NOTIFY '{' opt_notify_in_addr_list '}'",
"zone_option : T_DIALUP yea_or_nay",
"zone_option : error",
"master_in_addr_list : master_in_addr L_EOS",
"master_in_addr_list : master_in_addr_list master_in_addr L_EOS",
"master_in_addr : L_IPADDR",
"master_in_addr : L_IPADDR T_SEC_KEY key_ref",
"opt_notify_in_addr_list :",
"opt_notify_in_addr_list : notify_in_addr_list",
"notify_in_addr_list : notify_in_addr L_EOS",
"notify_in_addr_list : notify_in_addr_list notify_in_addr L_EOS",
"notify_in_addr : L_IPADDR",
"zone_forward_opt : T_ONLY",
"zone_forward_opt : T_FIRST",
"opt_zone_forwarders_list :",
"opt_zone_forwarders_list : zone_forwarders_in_addr_list",
"zone_forwarders_in_addr_list : zone_forwarders_in_addr L_EOS",
"zone_forwarders_in_addr_list : zone_forwarders_in_addr_list zone_forwarders_in_addr L_EOS",
"zone_forwarders_in_addr : L_IPADDR",
"trusted_keys_stmt : T_TRUSTED_KEYS '{' trusted_keys_list '}'",
"trusted_keys_list : trusted_key L_EOS",
"trusted_keys_list : trusted_keys_list trusted_key L_EOS",
"trusted_key : L_STRING L_NUMBER L_NUMBER L_NUMBER L_QSTRING",
"trusted_key : L_STRING L_STRING L_NUMBER L_NUMBER L_QSTRING",
"in_port : L_NUMBER",
"any_string : L_STRING",
"any_string : L_QSTRING",
};
#endif
#if YYDEBUG
#include <stdio.h>
#endif
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH 10000
#endif
#endif
#define YYINITSTACKSIZE 200
int yydebug;
int yynerrs;
int yyerrflag;
int yychar;
short *yyssp;
YYSTYPE *yyvsp;
YYSTYPE yyval;
YYSTYPE yylval;
short *yyss;
short *yysslim;
YYSTYPE *yyvs;
int yystacksize;
#line 1984 "ns_parser.y"

static char *
canonical_name(char *name) {
	char canonical[MAXDNAME];
	
	if (strlen(name) >= MAXDNAME)
		return (NULL);
	strcpy(canonical, name);
	if (makename(canonical, ".", sizeof canonical) < 0)
		return (NULL);
	return (savestr(canonical, 0));
}

static void
init_acls() {
	ip_match_element ime;
	ip_match_list iml;

	/* Create the predefined ACLs */

	/* ACL "any" */
	ime = new_ip_match_pattern(NULL, 0);
	iml = new_ip_match_list();
	add_to_ip_match_list(iml, ime);
	define_acl("any", iml);

	/* ACL "none" */
	ime = new_ip_match_pattern(NULL, 0);
	ip_match_negate(ime);
	iml = new_ip_match_list();
	add_to_ip_match_list(iml, ime);
	define_acl("none", iml);

	/* ACL "localhost" */
	ime = new_ip_match_localhost();
	iml = new_ip_match_list();
	add_to_ip_match_list(iml, ime);
	define_acl("localhost", iml);

	/* ACL "localnets" */
	ime = new_ip_match_localnets();
	iml = new_ip_match_list();
	add_to_ip_match_list(iml, ime);
	define_acl("localnets", iml);
}

static void
free_sym_value(int type, void *value) {
	ns_debug(ns_log_parser, 99, "free_sym_value: type %06x value %p",
		 type, value);
	type &= ~0xffff;
	switch (type) {
	case SYM_ACL:
		free_ip_match_list(value);
		break;
	case SYM_KEY:
		free_key_info(value);
		break;
	case SYM_CHANNEL:
		INSIST(log_free_channel(value) == 0);
		break;
	default:
		ns_panic(ns_log_parser, 1,
			 "unhandled case in free_sym_value()");
		/* NOTREACHED */
		break;
	}
}

static log_channel
lookup_channel(char *name) {
	symbol_value value;

	if (lookup_symbol(channeltab, name, SYM_CHANNEL, &value))
		return ((log_channel)(value.pointer));
	return (NULL);
}

static void
define_channel(const char *name, log_channel channel) {
	symbol_value value;

	value.pointer = channel;  
	INSIST(log_inc_references(channel) == 0);
	define_symbol(channeltab, name, SYM_CHANNEL, value, SYMBOL_FREE_VALUE);
}

static void
define_builtin_channels() {
	define_channel("default_syslog", syslog_channel);
	define_channel("default_debug", debug_channel);
	define_channel("default_stderr", stderr_channel);
	define_channel("null", null_channel);
}

static void
parser_setup() {
	seen_options = 0;
	logged_options_error = 0;
	seen_topology = 0;
	symtab = new_symbol_table(SYMBOL_TABLE_SIZE, NULL);
	if (authtab != NULL)
		free_symbol_table(authtab);
	authtab = new_symbol_table(AUTH_TABLE_SIZE, free_sym_value);
	if (channeltab != NULL)
		free_symbol_table(channeltab);
	channeltab = new_symbol_table(AUTH_TABLE_SIZE, free_sym_value);
	init_acls();
	define_builtin_channels();
	INIT_LIST(current_controls);
}

static void
parser_cleanup() {
	if (symtab != NULL)
		free_symbol_table(symtab);
	symtab = NULL;
	/*
	 * We don't clean up authtab here because the ip_match_lists are in
	 * use.
	 */
}

/*
 * Public Interface
 */

ip_match_list
lookup_acl(const char *name) {
	symbol_value value;

	if (lookup_symbol(authtab, name, SYM_ACL, &value))
		return ((ip_match_list)(value.pointer));
	return (NULL);
}

void
define_acl(const char *name, ip_match_list iml) {
	symbol_value value;

	INSIST(name != NULL);
	INSIST(iml != NULL);

	value.pointer = iml;
	define_symbol(authtab, name, SYM_ACL, value, SYMBOL_FREE_VALUE);
	ns_debug(ns_log_parser, 7, "acl %s", name);
	dprint_ip_match_list(ns_log_parser, iml, 2, "allow ", "deny ");
}

struct dst_key *
lookup_key(char *name) {
	symbol_value value;

	if (lookup_symbol(authtab, name, SYM_KEY, &value))
		return ((struct dst_key *)(value.pointer));
	return (NULL);
}

void
define_key(const char *name, struct dst_key *dst_key) {
	symbol_value value;

	INSIST(name != NULL);
	INSIST(dst_key != NULL);

	value.pointer = dst_key;
	define_symbol(authtab, name, SYM_KEY, value, SYMBOL_FREE_VALUE);
	dprint_key_info(dst_key);
}

time_t
parse_configuration(const char *filename) {
	FILE *config_stream;
	struct stat sb;

	config_stream = fopen(filename, "r");
	if (config_stream == NULL)
		ns_panic(ns_log_parser, 0, "can't open '%s'", filename);
	if (fstat(fileno(config_stream), &sb) == -1)
		ns_panic(ns_log_parser, 0, "can't stat '%s'", filename);

	lexer_setup();
	parser_setup();
	lexer_begin_file(filename, config_stream);
	(void)yyparse();
	lexer_end_file();
	parser_cleanup();
	return (sb.st_mtime);
}

void
parser_initialize(void) {
	lexer_initialize();
}

void
parser_shutdown(void) {
	if (authtab != NULL)
		free_symbol_table(authtab);
	if (channeltab != NULL)
		free_symbol_table(channeltab);
	lexer_shutdown();
}
#line 1310 "y.tab.c"
/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack()
{
    int newsize, i;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = yystacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;
    i = yyssp - yyss;
    newss = yyss ? (short *)realloc(yyss, newsize * sizeof *newss) :
      (short *)malloc(newsize * sizeof *newss);
    if (newss == NULL)
        return -1;
    yyss = newss;
    yyssp = newss + i;
    newvs = yyvs ? (YYSTYPE *)realloc(yyvs, newsize * sizeof *newvs) :
      (YYSTYPE *)malloc(newsize * sizeof *newvs);
    if (newvs == NULL)
        return -1;
    yyvs = newvs;
    yyvsp = newvs + i;
    yystacksize = newsize;
    yysslim = yyss + newsize - 1;
    return 0;
}

#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab

#ifndef YYPARSE_PARAM
#if defined(__cplusplus) || __STDC__
#define YYPARSE_PARAM_ARG void
#define YYPARSE_PARAM_DECL
#else	/* ! ANSI-C/C++ */
#define YYPARSE_PARAM_ARG
#define YYPARSE_PARAM_DECL
#endif	/* ANSI-C/C++ */
#else	/* YYPARSE_PARAM */
#ifndef YYPARSE_PARAM_TYPE
#define YYPARSE_PARAM_TYPE void *
#endif
#if defined(__cplusplus) || __STDC__
#define YYPARSE_PARAM_ARG YYPARSE_PARAM_TYPE YYPARSE_PARAM
#define YYPARSE_PARAM_DECL
#else	/* ! ANSI-C/C++ */
#define YYPARSE_PARAM_ARG YYPARSE_PARAM
#define YYPARSE_PARAM_DECL YYPARSE_PARAM_TYPE YYPARSE_PARAM;
#endif	/* ANSI-C/C++ */
#endif	/* ! YYPARSE_PARAM */

int
yyparse (YYPARSE_PARAM_ARG)
    YYPARSE_PARAM_DECL
{
    register int yym, yyn, yystate;
#if YYDEBUG
    register const char *yys;

    if ((yys = getenv("YYDEBUG")))
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = (-1);

    if (yyss == NULL && yygrowstack()) goto yyoverflow;
    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if ((yyn = yydefred[yystate])) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yysslim && yygrowstack())
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = (-1);
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;
#if defined(lint) || defined(__GNUC__)
    goto yynewerror;
#endif
yynewerror:
    yyerror("syntax error");
#if defined(lint) || defined(__GNUC__)
    goto yyerrlab;
#endif
yyerrlab:
    ++yynerrs;
yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yysslim && yygrowstack())
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = (-1);
        goto yyloop;
    }
yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    yyval = yyvsp[1-yym];
    switch (yyn)
    {
case 1:
#line 251 "ns_parser.y"
{
		if (EMPTY(current_controls))
			ns_ctl_defaults(&current_controls);
		ns_ctl_install(&current_controls);
	}
break;
case 16:
#line 277 "ns_parser.y"
{
		lexer_begin_file(yyvsp[-1].cp, NULL);
		(void)freestr(yyvsp[-1].cp);
	}
break;
case 17:
#line 288 "ns_parser.y"
{
		if (seen_options)
			parser_error(0, "cannot redefine options");
		current_options = new_options();
	}
break;
case 18:
#line 294 "ns_parser.y"
{
		if (!seen_options)
			set_options(current_options, 0);
		else
			free_options(current_options);
		current_options = NULL;
		seen_options = 1;
	}
break;
case 22:
#line 310 "ns_parser.y"
{
		if (current_options->hostname != NULL)
			(void)freestr(current_options->hostname);
		current_options->hostname = yyvsp[0].cp;
	}
break;
case 23:
#line 316 "ns_parser.y"
{
		if (current_options->version != NULL)
			(void)freestr(current_options->version);
		current_options->version = yyvsp[0].cp;
	}
break;
case 24:
#line 322 "ns_parser.y"
{
		if (current_options->directory != NULL)
			(void)freestr(current_options->directory);
		current_options->directory = yyvsp[0].cp;
	}
break;
case 25:
#line 328 "ns_parser.y"
{
		if (current_options->named_xfer != NULL)
			(void)freestr(current_options->named_xfer);
		current_options->named_xfer = yyvsp[0].cp;
	}
break;
case 26:
#line 334 "ns_parser.y"
{
		if (current_options->pid_filename != NULL)
			(void)freestr(current_options->pid_filename);
		current_options->pid_filename = yyvsp[0].cp;
	}
break;
case 27:
#line 340 "ns_parser.y"
{
		if (current_options->stats_filename != NULL)
			(void)freestr(current_options->stats_filename);
		current_options->stats_filename = yyvsp[0].cp;
	}
break;
case 28:
#line 346 "ns_parser.y"
{
		if (current_options->memstats_filename != NULL)
			(void)freestr(current_options->memstats_filename);
		current_options->memstats_filename = yyvsp[0].cp;
	}
break;
case 29:
#line 352 "ns_parser.y"
{
		if (current_options->dump_filename != NULL)
			(void)freestr(current_options->dump_filename);
		current_options->dump_filename = yyvsp[0].cp;
	}
break;
case 30:
#line 358 "ns_parser.y"
{
		current_options->preferred_glue =
			strcasecmp(yyvsp[0].cp, "aaaa") ? T_A : T_AAAA;
	}
break;
case 31:
#line 363 "ns_parser.y"
{
		set_global_boolean_option(current_options,
			OPTION_FAKE_IQUERY, yyvsp[0].num);
	}
break;
case 32:
#line 368 "ns_parser.y"
{
		set_global_boolean_option(current_options,
			OPTION_NORECURSE, !yyvsp[0].num);
	}
break;
case 33:
#line 373 "ns_parser.y"
{
		set_global_boolean_option(current_options,
			OPTION_NOFETCHGLUE, !yyvsp[0].num);
	}
break;
case 34:
#line 378 "ns_parser.y"
{
		set_global_boolean_option(current_options, 
			OPTION_HITCOUNT, yyvsp[0].num);
	}
break;
case 35:
#line 383 "ns_parser.y"
{
		current_options->notify = notify_explicit;
	}
break;
case 36:
#line 387 "ns_parser.y"
{
		if (yyvsp[0].num)
			current_options->notify = notify_yes;
		else
			current_options->notify = notify_no;
	}
break;
case 37:
#line 394 "ns_parser.y"
{
		if (initial_configuration && yyvsp[0].num)
			ns_notice(ns_log_default,
				  "suppressing initial notifies");
		set_global_boolean_option(current_options, 
			OPTION_SUPNOTIFY_INITIAL, yyvsp[0].num);
	}
break;
case 38:
#line 402 "ns_parser.y"
{
		set_global_boolean_option(current_options,
			OPTION_HOSTSTATS, yyvsp[0].num);
	}
break;
case 39:
#line 407 "ns_parser.y"
{
		set_global_boolean_option(current_options,
			OPTION_DEALLOC_ON_EXIT, yyvsp[0].num);
	}
break;
case 40:
#line 412 "ns_parser.y"
{
		set_global_boolean_option(current_options, OPTION_USE_IXFR, yyvsp[0].num);
	}
break;
case 41:
#line 416 "ns_parser.y"
{
		set_global_boolean_option(current_options,
					  OPTION_MAINTAIN_IXFR_BASE, yyvsp[0].num);
	}
break;
case 42:
#line 421 "ns_parser.y"
{
		set_global_boolean_option(current_options,
					  OPTION_NORFC2308_TYPE1, yyvsp[0].num);
		set_global_boolean_option(current_options,
					  OPTION_NONAUTH_NXDOMAIN, !yyvsp[0].num);
	}
break;
case 43:
#line 428 "ns_parser.y"
{
		set_global_boolean_option(current_options, OPTION_NONAUTH_NXDOMAIN,
				   !yyvsp[0].num);
	}
break;
case 44:
#line 433 "ns_parser.y"
{
		set_global_boolean_option(current_options,
			OPTION_MULTIPLE_CNAMES, yyvsp[0].num);
	}
break;
case 45:
#line 438 "ns_parser.y"
{
		current_options->check_names[yyvsp[-1].s_int] = (enum severity)yyvsp[0].s_int;
	}
break;
case 46:
#line 442 "ns_parser.y"
{
		set_global_boolean_option(current_options,
					  OPTION_USE_ID_POOL, yyvsp[0].num);
	}
break;
case 47:
#line 447 "ns_parser.y"
{
		set_global_boolean_option(current_options,
                        		  OPTION_NORFC2308_TYPE1, !yyvsp[0].num);
	}
break;
case 48:
#line 452 "ns_parser.y"
{
		char port_string[20];
		symbol_value value;

		(void)sprintf(port_string, "%u/%u", AF_INET, yyvsp[-3].us_int);
		if (lookup_symbol(symtab, port_string, SYM_PORT, NULL))
			parser_error(0,
				     "cannot redefine listen-on for port %u",
				     ntohs(yyvsp[-3].us_int));
		else {
			add_listen_on(current_options, yyvsp[-3].us_int, yyvsp[-1].iml, AF_INET);
			value.pointer = NULL;
			define_symbol(symtab, port_string, SYM_PORT, value, 0);
		}

	}
break;
case 49:
#line 469 "ns_parser.y"
{
		char port_string[20];
		symbol_value value;

		(void)sprintf(port_string, "%u/%u", AF_INET6, yyvsp[-3].us_int);
		if (lookup_symbol(symtab, port_string, SYM_PORT, NULL))
			parser_error(0,
				     "cannot redefine listen-on for port %u",
				     ntohs(yyvsp[-3].us_int));
		else {
			add_listen_on(current_options, yyvsp[-3].us_int, yyvsp[-1].iml, AF_INET6);
			value.pointer = NULL;
			define_symbol(symtab, port_string, SYM_PORT, value, 0);
		}

	}
break;
case 51:
#line 487 "ns_parser.y"
{
		if (current_options->fwdtab) {
			free_forwarders(current_options->fwdtab);
			current_options->fwdtab = NULL;
		}
	}
break;
case 55:
#line 497 "ns_parser.y"
{
		struct sockaddr_in *sin =
			satosin(sstosa(&current_options->axfr_src));

		if (ns_family(&yyvsp[0].ip_addr) == 0) {
			memset(sin, 0, sizeof(*sin));
		} else if (ns_family(&yyvsp[0].ip_addr) == AF_INET) {
			memcpy(sin, &yyvsp[0].ip_addr, sa_slen(sstosa(&yyvsp[0].ip_addr)));
		} else {
			parser_error(0, "invalid ipv4 address %s",
				sa_ntoa(sstosa(&yyvsp[0].ip_addr)));
		}
	}
break;
case 56:
#line 511 "ns_parser.y"
{
		struct sockaddr_in6 *sin6 =
			satosin6(sstosa(&current_options->axfr_src_v6));

		if (ns_family(&yyvsp[0].ip_addr) == 0) {
			memset(sin6, 0, sizeof(*sin6));
		} else if (ns_family(&yyvsp[0].ip_addr) == AF_INET6) {
			memcpy(sin6, &yyvsp[0].ip_addr, sa_slen(sstosa(&yyvsp[0].ip_addr)));
		} else {
			parser_error(0, "invalid ipv6 address %s",
				sa_ntoa(sstosa(&yyvsp[0].ip_addr)));
		}
	}
break;
case 57:
#line 525 "ns_parser.y"
{
		if (current_options->query_acl) {
			parser_warning(0,
			      "options allow-query acl already set; skipping");
			free_ip_match_list(yyvsp[-1].iml);
		} else 
			current_options->query_acl = yyvsp[-1].iml;
	}
break;
case 58:
#line 534 "ns_parser.y"
{
		if (current_options->recursion_acl) {
			parser_warning(0,
			      "options allow-recursion acl already set; skipping");
			free_ip_match_list(yyvsp[-1].iml);
		} else
			current_options->recursion_acl = yyvsp[-1].iml;
	}
break;
case 59:
#line 543 "ns_parser.y"
{
		if (current_options->transfer_acl) {
			parser_warning(0,
			   "options allow-transfer acl already set; skipping");
			free_ip_match_list(yyvsp[-1].iml);
		} else 
			current_options->transfer_acl = yyvsp[-1].iml;
	}
break;
case 60:
#line 552 "ns_parser.y"
{
		if (current_options->sortlist) {
			parser_warning(0,
			      "options sortlist already set; skipping");
			free_ip_match_list(yyvsp[-1].iml);
		} else
			current_options->sortlist = yyvsp[-1].iml;
	}
break;
case 61:
#line 561 "ns_parser.y"
{
		if (current_options->also_notify) {
			parser_warning(0,
			    "duplicate also-notify clause: overwriting");
			free_also_notify(current_options);
			current_options->also_notify = NULL;
		}
	}
break;
case 63:
#line 571 "ns_parser.y"
{
		if (current_options->blackhole_acl) {
			parser_warning(0,
			      "options blackhole already set; skipping");
			free_ip_match_list(yyvsp[-1].iml);
		} else
			current_options->blackhole_acl = yyvsp[-1].iml;
	}
break;
case 64:
#line 580 "ns_parser.y"
{
		if (current_options->topology) {
			parser_warning(0,
			      "options topology already set; skipping");
			free_ip_match_list(yyvsp[-1].iml);
		} else
			current_options->topology = yyvsp[-1].iml;
	}
break;
case 65:
#line 589 "ns_parser.y"
{
		/* To get around the $$ = $1 default rule. */
	}
break;
case 67:
#line 594 "ns_parser.y"
{
		current_options->transfer_format = yyvsp[0].axfr_fmt;
	}
break;
case 68:
#line 598 "ns_parser.y"
{
		current_options->max_transfer_time_in = yyvsp[0].num * 60;
	}
break;
case 69:
#line 602 "ns_parser.y"
{
		current_options->serial_queries = yyvsp[0].num;
	}
break;
case 70:
#line 606 "ns_parser.y"
{
		current_options->clean_interval = yyvsp[0].num * 60;
	}
break;
case 71:
#line 610 "ns_parser.y"
{
		current_options->interface_interval = yyvsp[0].num * 60;
	}
break;
case 72:
#line 614 "ns_parser.y"
{
		current_options->stats_interval = yyvsp[0].num * 60;
	}
break;
case 73:
#line 618 "ns_parser.y"
{
		current_options->max_host_stats = yyvsp[0].num;
	}
break;
case 74:
#line 622 "ns_parser.y"
{
		current_options->max_log_size_ixfr = yyvsp[0].ul_int;
	}
break;
case 75:
#line 626 "ns_parser.y"
{
		current_options->max_ncache_ttl = yyvsp[0].num;
	}
break;
case 76:
#line 630 "ns_parser.y"
{
		current_options->lame_ttl = yyvsp[0].num;
	}
break;
case 77:
#line 634 "ns_parser.y"
{
		current_options->heartbeat_interval = yyvsp[0].num * 60;
	}
break;
case 78:
#line 638 "ns_parser.y"
{
		set_global_boolean_option(current_options,
                                          OPTION_NODIALUP, !yyvsp[0].num);
	}
break;
case 79:
#line 643 "ns_parser.y"
{
		if (current_options->ordering)
			free_rrset_order_list(current_options->ordering);
		current_options->ordering = yyvsp[-1].rol;
	}
break;
case 80:
#line 649 "ns_parser.y"
{
		set_global_boolean_option(current_options,
					  OPTION_TREAT_CR_AS_SPACE, yyvsp[0].num);
	}
break;
case 81:
#line 654 "ns_parser.y"
{
		if (yyvsp[0].num >= 1)
			current_options->minroots = yyvsp[0].num;
	}
break;
case 82:
#line 659 "ns_parser.y"
{
		if (yyvsp[0].num < 512)
			current_options->edns_udp_size = 512;
		else if (yyvsp[0].num > EDNS_MESSAGE_SZ)
			current_options->edns_udp_size = EDNS_MESSAGE_SZ;
		else
			current_options->edns_udp_size = yyvsp[0].num;
	}
break;
case 88:
#line 683 "ns_parser.y"
{
		ns_ctl_add(&current_controls, ns_ctl_new_inet(sstosa(&yyvsp[-6].ip_addr), yyvsp[-4].us_int, yyvsp[-1].iml));
	}
break;
case 89:
#line 689 "ns_parser.y"
{
		parser_warning(0, "Ignoring BIND 9 inet control clause");
		free_ip_match_list(yyvsp[-5].iml);
	}
break;
case 90:
#line 696 "ns_parser.y"
{
		parser_warning(0, "Ignoring BIND 9 inet control clause");
		free_ip_match_list(yyvsp[-5].iml);
	}
break;
case 91:
#line 701 "ns_parser.y"
{
#ifndef NO_SOCKADDR_UN
		ns_ctl_add(&current_controls, ns_ctl_new_unix(yyvsp[-6].cp, yyvsp[-4].num, yyvsp[-2].num, yyvsp[0].num));
#endif
		freestr(yyvsp[-6].cp);
	}
break;
case 93:
#line 711 "ns_parser.y"
{
		rrset_order_list rol;

		rol = new_rrset_order_list();
		if (yyvsp[-1].roe != NULL) {
			add_to_rrset_order_list(rol, yyvsp[-1].roe);
		}
		
		yyval.rol = rol;
	}
break;
case 94:
#line 722 "ns_parser.y"
{
		if (yyvsp[-1].roe != NULL) {
			add_to_rrset_order_list(yyvsp[-2].rol, yyvsp[-1].roe);
		}
		yyval.rol = yyvsp[-2].rol;
	}
break;
case 95:
#line 731 "ns_parser.y"
{
		yyval.s_int = C_ANY;
	}
break;
case 96:
#line 735 "ns_parser.y"
{
		symbol_value value;

		if (lookup_symbol(constants, yyvsp[0].cp, SYM_CLASS, &value))
			yyval.s_int = value.integer;
		else {
			parser_error(0, "unknown class '%s'; using ANY", yyvsp[0].cp);
			yyval.s_int = C_ANY;
		}
		(void)freestr(yyvsp[0].cp);
	}
break;
case 97:
#line 749 "ns_parser.y"
{
		yyval.s_int = ns_t_any;
	}
break;
case 98:
#line 753 "ns_parser.y"
{
		int success;

		if (strcmp(yyvsp[0].cp, "*") == 0) {
			yyval.s_int = ns_t_any;
		} else {
			yyval.s_int = __sym_ston(__p_type_syms, yyvsp[0].cp, &success);
			if (success == 0) {
				yyval.s_int = ns_t_any;
				parser_error(0,
					     "unknown type '%s'; assuming ANY",
					     yyvsp[0].cp);
			}
		}
		(void)freestr(yyvsp[0].cp);
	}
break;
case 99:
#line 772 "ns_parser.y"
{
		yyval.cp = savestr("*", 1);
	}
break;
case 100:
#line 776 "ns_parser.y"
{
		if (strcmp(".",yyvsp[0].cp) == 0 || strcmp("*.",yyvsp[0].cp) == 0) {
			yyval.cp = savestr("*", 1);
			(void)freestr(yyvsp[0].cp);
		} else {
			yyval.cp = yyvsp[0].cp ;
		}
		/* XXX Should do any more name validation here? */
	}
break;
case 101:
#line 788 "ns_parser.y"
{
		enum ordering o;

		if (strlen(yyvsp[0].cp) == 0) {
			parser_error(0, "null order name");
			yyval.roe = NULL ;
		} else {
			o = lookup_ordering(yyvsp[0].cp);
			if (o == unknown_order) {
				o = (enum ordering)DEFAULT_ORDERING;
				parser_error(0,
					     "invalid order name '%s'; using %s",
					     yyvsp[0].cp, p_order(o));
			}
			
			(void)freestr(yyvsp[0].cp);
			
			yyval.roe = new_rrset_order_element(yyvsp[-4].s_int, yyvsp[-3].s_int, yyvsp[-2].cp, o);
		}
	}
break;
case 102:
#line 811 "ns_parser.y"
{
		yyval.axfr_fmt = axfr_one_answer;
	}
break;
case 103:
#line 815 "ns_parser.y"
{
		yyval.axfr_fmt = axfr_many_answers;
	}
break;
case 104:
#line 820 "ns_parser.y"
{ yyval.ip_addr = yyvsp[0].ip_addr; }
break;
case 105:
#line 821 "ns_parser.y"
{ ns_family(&yyval.ip_addr) = 0; }
break;
case 106:
#line 824 "ns_parser.y"
{ yyval.us_int = yyvsp[0].us_int; }
break;
case 107:
#line 825 "ns_parser.y"
{ yyval.us_int = htons(0); }
break;
case 108:
#line 829 "ns_parser.y"
{
		u_int port;
		struct sockaddr_in *sin =
			satosin(sstosa(&current_options->query_source));

		port = sin->sin_port;
		if (ns_family(&yyvsp[0].ip_addr) == 0) {
			memset(sin, 0, sizeof(*sin));
			sin->sin_family = AF_INET;
#ifdef HAVE_SA_LEN
			sin->sin_len = sizeof(*sin);
#endif
			sin->sin_addr.s_addr = htonl(INADDR_ANY);
		} else if (ns_family(&yyvsp[0].ip_addr) == AF_INET) {
			memcpy(sin, &yyvsp[0].ip_addr, sa_slen(sstosa(&yyvsp[0].ip_addr)));
		} else {
			parser_error(0, "invalid ipv4 address %s",
				sa_ntoa(sstosa(&yyvsp[0].ip_addr)));
		}
		sin->sin_port = port;
	}
break;
case 109:
#line 853 "ns_parser.y"
{
		satosin(sstosa(&current_options->query_source))->sin_port = yyvsp[0].us_int;
	}
break;
case 114:
#line 865 "ns_parser.y"
{
		u_int port;
		struct sockaddr_in6 *sin6 =
			satosin6(sstosa(&current_options->query_source_v6));

		port = sin6->sin6_port;
		if (ns_family(&yyvsp[0].ip_addr) == 0) {
			memset(sin6, 0, sizeof(*sin6));
			sin6->sin6_family = AF_INET6;
#ifdef HAVE_SA_LEN
			sin6->sin6_len = sizeof(*sin6);
#endif
			/* XXX SHOULD USE getaddrinfo */
		} else if (ns_family(&yyvsp[0].ip_addr) == AF_INET6) {
			memcpy(sin6, &yyvsp[0].ip_addr, sa_slen(sstosa(&yyvsp[0].ip_addr)));
		} else {
			parser_error(0, "invalid ipv6 address %s",
				sa_ntoa(sstosa(&yyvsp[0].ip_addr)));
		}
		sin6->sin6_port = port;
	}
break;
case 115:
#line 889 "ns_parser.y"
{
		satosin6(sstosa(&current_options->query_source_v6))->sin6_port
		    = yyvsp[0].us_int;
	}
break;
case 120:
#line 901 "ns_parser.y"
{ yyval.us_int = htons(NS_DEFAULTPORT); }
break;
case 121:
#line 902 "ns_parser.y"
{ yyval.us_int = yyvsp[0].us_int; }
break;
case 122:
#line 905 "ns_parser.y"
{ yyval.us_int = htons(0); }
break;
case 123:
#line 906 "ns_parser.y"
{ yyval.us_int = yyvsp[0].us_int; }
break;
case 124:
#line 911 "ns_parser.y"
{ 
		yyval.num = 1;	
	}
break;
case 125:
#line 915 "ns_parser.y"
{ 
		yyval.num = 1;	
	}
break;
case 126:
#line 919 "ns_parser.y"
{ 
		yyval.num = 0;	
	}
break;
case 127:
#line 923 "ns_parser.y"
{ 
		yyval.num = 0;	
	}
break;
case 128:
#line 927 "ns_parser.y"
{ 
		if (yyvsp[0].num == 1 || yyvsp[0].num == 0) {
			yyval.num = yyvsp[0].num;
		} else {
			parser_warning(0,
				       "number should be 0 or 1; assuming 1");
			yyval.num = 1;
		}
	}
break;
case 129:
#line 939 "ns_parser.y"
{
		yyval.s_int = primary_trans;
	}
break;
case 130:
#line 943 "ns_parser.y"
{
		yyval.s_int = secondary_trans;
	}
break;
case 131:
#line 947 "ns_parser.y"
{
		yyval.s_int = response_trans;
	}
break;
case 132:
#line 953 "ns_parser.y"
{
		yyval.s_int = warn;
	}
break;
case 133:
#line 957 "ns_parser.y"
{
		yyval.s_int = fail;
	}
break;
case 134:
#line 961 "ns_parser.y"
{
		yyval.s_int = ignore;
	}
break;
case 135:
#line 967 "ns_parser.y"
{
		set_global_boolean_option(current_options,
			OPTION_FORWARD_ONLY, 1);
	}
break;
case 136:
#line 972 "ns_parser.y"
{
		set_global_boolean_option(current_options,
			OPTION_FORWARD_ONLY, 0);
	}
break;
case 137:
#line 977 "ns_parser.y"
{
		parser_warning(0, "forward if-no-answer is unimplemented");
	}
break;
case 138:
#line 981 "ns_parser.y"
{
		parser_warning(0, "forward if-no-domain is unimplemented");
	}
break;
case 139:
#line 987 "ns_parser.y"
{
		current_options->data_size = yyvsp[0].ul_int;
	}
break;
case 140:
#line 991 "ns_parser.y"
{
		current_options->stack_size = yyvsp[0].ul_int;
	}
break;
case 141:
#line 995 "ns_parser.y"
{
		current_options->core_size = yyvsp[0].ul_int;
	}
break;
case 142:
#line 999 "ns_parser.y"
{
		current_options->files = yyvsp[0].ul_int;
	}
break;
case 143:
#line 1005 "ns_parser.y"
{
		u_long result;

		if (unit_to_ulong(yyvsp[0].cp, &result))
			yyval.ul_int = result;
		else {
			parser_error(0, "invalid unit string '%s'", yyvsp[0].cp);
			/* 0 means "use default" */
			yyval.ul_int = 0;
		}
		(void)freestr(yyvsp[0].cp);
	}
break;
case 144:
#line 1018 "ns_parser.y"
{	
		yyval.ul_int = (u_long)yyvsp[0].num;
	}
break;
case 145:
#line 1022 "ns_parser.y"
{
		yyval.ul_int = 0;
	}
break;
case 146:
#line 1026 "ns_parser.y"
{
		yyval.ul_int = ULONG_MAX;
	}
break;
case 147:
#line 1032 "ns_parser.y"
{
		current_options->transfers_in = (u_long) yyvsp[0].num;
	}
break;
case 148:
#line 1036 "ns_parser.y"
{
		current_options->transfers_out = (u_long) yyvsp[0].num;
	}
break;
case 149:
#line 1040 "ns_parser.y"
{
		current_options->transfers_per_ns = (u_long) yyvsp[0].num;
	}
break;
case 152:
#line 1050 "ns_parser.y"
{
		/* nothing */
	}
break;
case 153:
#line 1054 "ns_parser.y"
{
		/* nothing */
	}
break;
case 154:
#line 1060 "ns_parser.y"
{
	  	add_global_forwarder(current_options, sstosa(&yyvsp[0].ip_addr));
	}
break;
case 157:
#line 1070 "ns_parser.y"
{
		/* nothing */
	}
break;
case 158:
#line 1074 "ns_parser.y"
{
		/* nothing */
	}
break;
case 159:
#line 1080 "ns_parser.y"
{
	  	add_global_also_notify(current_options, sstosa(&yyvsp[0].ip_addr));
	}
break;
case 160:
#line 1090 "ns_parser.y"
{
		current_logging = begin_logging();
	}
break;
case 161:
#line 1094 "ns_parser.y"
{
		end_logging(current_logging, 1);
		current_logging = NULL;
	}
break;
case 165:
#line 1106 "ns_parser.y"
{
		current_category = yyvsp[0].s_int;
	}
break;
case 167:
#line 1111 "ns_parser.y"
{
		chan_type = log_null;
		chan_flags = 0;
		chan_level = log_info;
	}
break;
case 168:
#line 1117 "ns_parser.y"
{
		log_channel current_channel = NULL;

		if (lookup_channel(yyvsp[-4].cp) != NULL) {
			parser_error(0, "can't redefine channel '%s'", yyvsp[-4].cp);
		} else {
			switch (chan_type) {
			case log_file:
				current_channel =
					log_new_file_channel(chan_flags,
							     chan_level,
							     chan_name, NULL,
							     chan_versions,
							     chan_max_size);
				log_set_file_owner(current_channel,
						   user_id, group_id);
				chan_name = freestr(chan_name);
				break;
			case log_syslog:
				current_channel =
					log_new_syslog_channel(chan_flags,
							       chan_level,
							       chan_facility);
				break;
			case log_null:
				current_channel = log_new_null_channel();
				break;
			default:
				ns_panic(ns_log_parser, 1,
					 "unknown channel type: %d",
					 chan_type);
			}
			if (current_channel == NULL)
				ns_panic(ns_log_parser, 0,
					 "couldn't create channel");
			define_channel(yyvsp[-4].cp, current_channel);
		}
		(void)freestr(yyvsp[-4].cp);
	}
break;
case 169:
#line 1159 "ns_parser.y"
{
		symbol_value value;

		if (lookup_symbol(constants, yyvsp[0].cp, SYM_LOGGING, &value)) {
			chan_level = value.integer;
		} else {
			parser_error(0, "unknown severity '%s'", yyvsp[0].cp);
			chan_level = log_debug(99);
		}
		(void)freestr(yyvsp[0].cp);
	}
break;
case 170:
#line 1171 "ns_parser.y"
{
		chan_level = log_debug(1);
	}
break;
case 171:
#line 1175 "ns_parser.y"
{
		chan_level = yyvsp[0].num;
	}
break;
case 172:
#line 1179 "ns_parser.y"
{
		chan_level = 0;
		chan_flags |= LOG_USE_CONTEXT_LEVEL|LOG_REQUIRE_DEBUG;
	}
break;
case 173:
#line 1186 "ns_parser.y"
{
		chan_versions = yyvsp[0].num;
	}
break;
case 174:
#line 1190 "ns_parser.y"
{
		chan_versions = LOG_MAX_VERSIONS;
	}
break;
case 175:
#line 1196 "ns_parser.y"
{
		chan_max_size = yyvsp[0].ul_int;
	}
break;
case 176:
#line 1202 "ns_parser.y"
{
		chan_versions = 0;
		chan_max_size = ULONG_MAX;
	}
break;
case 177:
#line 1207 "ns_parser.y"
{
		chan_max_size = ULONG_MAX;
	}
break;
case 178:
#line 1211 "ns_parser.y"
{
		chan_versions = 0;
	}
break;
case 181:
#line 1219 "ns_parser.y"
{
		chan_flags |= LOG_CLOSE_STREAM;
		chan_type = log_file;
		chan_name = yyvsp[-1].cp;
	}
break;
case 182:
#line 1227 "ns_parser.y"
{ yyval.cp = yyvsp[0].cp; }
break;
case 183:
#line 1228 "ns_parser.y"
{ yyval.cp = savestr("syslog", 1); }
break;
case 184:
#line 1231 "ns_parser.y"
{ yyval.s_int = LOG_DAEMON; }
break;
case 185:
#line 1233 "ns_parser.y"
{
		symbol_value value;

		if (lookup_symbol(constants, yyvsp[0].cp, SYM_SYSLOG, &value)) {
			yyval.s_int = value.integer;
		} else {
			parser_error(0, "unknown facility '%s'", yyvsp[0].cp);
			yyval.s_int = LOG_DAEMON;
		}
		(void)freestr(yyvsp[0].cp);
	}
break;
case 186:
#line 1247 "ns_parser.y"
{
		chan_type = log_syslog;
		chan_facility = yyvsp[0].s_int;
	}
break;
case 187:
#line 1253 "ns_parser.y"
{ /* nothing to do */ }
break;
case 188:
#line 1254 "ns_parser.y"
{ /* nothing to do */ }
break;
case 189:
#line 1256 "ns_parser.y"
{
		chan_type = log_null;
	}
break;
case 190:
#line 1259 "ns_parser.y"
{ /* nothing to do */ }
break;
case 191:
#line 1261 "ns_parser.y"
{
		if (yyvsp[0].num)
			chan_flags |= LOG_TIMESTAMP;
		else
			chan_flags &= ~LOG_TIMESTAMP;
	}
break;
case 192:
#line 1268 "ns_parser.y"
{
		if (yyvsp[0].num)
			chan_flags |= LOG_PRINT_CATEGORY;
		else
			chan_flags &= ~LOG_PRINT_CATEGORY;
	}
break;
case 193:
#line 1275 "ns_parser.y"
{
		if (yyvsp[0].num)
			chan_flags |= LOG_PRINT_LEVEL;
		else
			chan_flags &= ~LOG_PRINT_LEVEL;
	}
break;
case 198:
#line 1289 "ns_parser.y"
{ yyval.cp = savestr("null", 1); }
break;
case 199:
#line 1293 "ns_parser.y"
{
		log_channel channel;

		if (current_category >= 0) {
			channel = lookup_channel(yyvsp[0].cp);
			if (channel != NULL) {
				add_log_channel(current_logging,
						current_category, channel);
			} else
				parser_error(0, "unknown channel '%s'", yyvsp[0].cp);
		}
		(void)freestr(yyvsp[0].cp);
	}
break;
case 204:
#line 1314 "ns_parser.y"
{ yyval.cp = savestr("default", 1); }
break;
case 205:
#line 1315 "ns_parser.y"
{ yyval.cp = savestr("notify", 1); }
break;
case 206:
#line 1319 "ns_parser.y"
{
		symbol_value value;

		if (lookup_symbol(constants, yyvsp[0].cp, SYM_CATEGORY, &value))
			yyval.s_int = value.integer;
		else {
			parser_error(0, "invalid logging category '%s'",
				     yyvsp[0].cp);
			yyval.s_int = -1;
		}
		(void)freestr(yyvsp[0].cp);
	}
break;
case 207:
#line 1338 "ns_parser.y"
{
		char ip_printable[NI_MAXHOST];
		symbol_value value;
		
		getnameinfo(sstosa(&yyvsp[0].ip_addr), sa_slen(sstosa(&yyvsp[0].ip_addr)), ip_printable,
			sizeof(ip_printable), NULL, 0, NI_NUMERICHOST);
		value.pointer = NULL;
		if (lookup_symbol(symtab, ip_printable, SYM_SERVER, NULL))
			seen_server = 1;
		else
			seen_server = 0;
		if (seen_server)
			parser_error(0, "cannot redefine server '%s'", 
				     ip_printable);
		else
			define_symbol(symtab, ip_printable, SYM_SERVER, value,
				      0);
		current_server = begin_server(sstosa(&yyvsp[0].ip_addr));
	}
break;
case 208:
#line 1358 "ns_parser.y"
{
		end_server(current_server, !seen_server);
	}
break;
case 211:
#line 1368 "ns_parser.y"
{
		set_server_option(current_server, SERVER_INFO_BOGUS, yyvsp[0].num);
	}
break;
case 212:
#line 1372 "ns_parser.y"
{
		set_server_option(current_server, SERVER_INFO_SUPPORT_IXFR, yyvsp[0].num);
	}
break;
case 213:
#line 1376 "ns_parser.y"
{
		set_server_transfers(current_server, (int)yyvsp[0].num);
	}
break;
case 214:
#line 1380 "ns_parser.y"
{
		set_server_transfer_format(current_server, yyvsp[0].axfr_fmt);
	}
break;
case 216:
#line 1385 "ns_parser.y"
{
		set_server_option(current_server, SERVER_INFO_EDNS, yyvsp[0].num);
	}
break;
case 218:
#line 1396 "ns_parser.y"
{
		ip_match_list iml;
		
		iml = new_ip_match_list();
		if (yyvsp[-1].ime != NULL)
			add_to_ip_match_list(iml, yyvsp[-1].ime);
		yyval.iml = iml;
	}
break;
case 219:
#line 1405 "ns_parser.y"
{
		if (yyvsp[-1].ime != NULL)
			add_to_ip_match_list(yyvsp[-2].iml, yyvsp[-1].ime);
		yyval.iml = yyvsp[-2].iml;
	}
break;
case 221:
#line 1414 "ns_parser.y"
{
		if (yyvsp[0].ime != NULL)
			ip_match_negate(yyvsp[0].ime);
		yyval.ime = yyvsp[0].ime;
	}
break;
case 222:
#line 1420 "ns_parser.y"
{
		char *key_name;
		struct dst_key *dst_key;

		key_name = canonical_name(yyvsp[0].cp);
		if (key_name == NULL) {
			parser_error(0, "can't make key name '%s' canonical",
				     yyvsp[0].cp);
			key_name = savestr("__bad_key__", 1);
		}
		dst_key = find_key(key_name, NULL);
		if (dst_key == NULL) {
			parser_error(0, "key \"%s\" not found", key_name);
			yyval.ime = NULL;
		}
		else
			yyval.ime = new_ip_match_key(dst_key);
	        (void)freestr(key_name);
		freestr(yyvsp[0].cp);
	}
break;
case 223:
#line 1443 "ns_parser.y"
{
		yyval.ime = new_ip_match_pattern(sstosa(&yyvsp[0].ip_addr), ~0);
	}
break;
case 224:
#line 1447 "ns_parser.y"
{
		int maxlen = (ns_family(&yyvsp[-2].ip_addr) == AF_INET6 ? 128 : 32);
		if (yyvsp[0].num < 0 || yyvsp[0].num > maxlen) {
			parser_error(0, "mask bits out of range; skipping");
			yyval.ime = NULL;
		} else {
			yyval.ime = new_ip_match_pattern(sstosa(&yyvsp[-2].ip_addr), yyvsp[0].num);
			if (yyval.ime == NULL)
				parser_error(0, 
					   "address/mask mismatch; skipping");
		}
	}
break;
case 225:
#line 1460 "ns_parser.y"
{
		struct sockaddr_in sin;

		if (yyvsp[-2].num > 255) {
			parser_error(0, "address out of range; skipping");
			yyval.ime = NULL;
		} else {
			if (yyvsp[0].num < 0 || yyvsp[0].num > 32) {
				parser_error(0,
					"mask bits out of range; skipping");
					yyval.ime = NULL;
			} else {
				memset(&sin, 0, sizeof(sin));
				sin.sin_family = AF_INET;
#ifdef HAVE_SA_LEN
				sin.sin_len = sizeof(sin);
#endif
				sin.sin_addr.s_addr = htonl((yyvsp[-2].num & 0xff) << 24);
				yyval.ime = new_ip_match_pattern((struct sockaddr *)&sin, yyvsp[0].num);
				if (yyval.ime == NULL)
					parser_error(0, 
					   "address/mask mismatch; skipping");
			}
		}
	}
break;
case 227:
#line 1487 "ns_parser.y"
{
		char name[256];

		/*
		 * We want to be able to clean up this iml later so
		 * we give it a name and treat it like any other acl.
		 */
		sprintf(name, "__internal_%p", yyvsp[-1].iml);
		define_acl(name, yyvsp[-1].iml);
  		yyval.ime = new_ip_match_indirect(yyvsp[-1].iml);
	}
break;
case 228:
#line 1501 "ns_parser.y"
{
		ip_match_list iml;

		iml = lookup_acl(yyvsp[0].cp);
		if (iml == NULL) {
			parser_error(0, "unknown ACL '%s'", yyvsp[0].cp);
			yyval.ime = NULL;
		} else
			yyval.ime = new_ip_match_indirect(iml);
		(void)freestr(yyvsp[0].cp);
	}
break;
case 229:
#line 1519 "ns_parser.y"
{
		struct dst_key *dst_key;
		char *key_name;

		key_name = canonical_name(yyvsp[0].cp);
		if (key_name == NULL) {
			parser_error(0, "can't make key name '%s' canonical",
				     yyvsp[0].cp);
			yyval.keyi = NULL;
		} else {
			dst_key = lookup_key(key_name);
			if (dst_key == NULL) {
				parser_error(0, "unknown key '%s'", key_name);
				yyval.keyi = NULL;
			} else
				yyval.keyi = dst_key;
			key_name = freestr(key_name);
		}
		(void)freestr(yyvsp[0].cp);
	}
break;
case 230:
#line 1542 "ns_parser.y"
{
		if (yyvsp[0].keyi == NULL)
			parser_error(0, "empty key not added to server list ");
		else
			add_server_key_info(current_server, yyvsp[0].keyi);
	}
break;
case 234:
#line 1555 "ns_parser.y"
{ /* empty */ }
break;
case 238:
#line 1563 "ns_parser.y"
{
		current_algorithm = NULL;
		current_secret = NULL;
	}
break;
case 239:
#line 1568 "ns_parser.y"
{
		struct dst_key *dst_key;
		char *key_name;

		key_name = canonical_name(yyvsp[-3].cp);
		if (key_name == NULL) {
			parser_error(0, "can't make key name '%s' canonical",
				     yyvsp[-3].cp);
		} else if (lookup_key(key_name) != NULL) {
			parser_error(0, "can't redefine key '%s'", key_name);
		} else {
			if (current_algorithm == NULL ||
			    current_secret == NULL)  {
				parser_error(0, "skipping bad key '%s'",
					     key_name);
			} else {
				dst_key = new_key_info(key_name,
						       current_algorithm,
						       current_secret);
				if (dst_key != NULL) {
					define_key(key_name, dst_key);
					if (secretkey_info == NULL)
						secretkey_info =
							new_key_info_list();
					add_to_key_info_list(secretkey_info,
							     dst_key);
				}
			}
		}
		if (key_name != NULL)
			key_name = freestr(key_name);
		if (current_algorithm != NULL)
			current_algorithm = freestr(current_algorithm);
		if (current_secret != NULL)
			current_secret = freestr(current_secret);
		(void)freestr(yyvsp[-3].cp);
	}
break;
case 240:
#line 1608 "ns_parser.y"
{
		current_algorithm = yyvsp[-1].cp;
		current_secret = yyvsp[0].cp;
	}
break;
case 241:
#line 1613 "ns_parser.y"
{
		current_algorithm = yyvsp[0].cp;
		current_secret = yyvsp[-1].cp;
	}
break;
case 242:
#line 1618 "ns_parser.y"
{
		current_algorithm = NULL;
		current_secret = NULL;
	}
break;
case 243:
#line 1624 "ns_parser.y"
{ yyval.cp = yyvsp[-1].cp; }
break;
case 244:
#line 1627 "ns_parser.y"
{ yyval.cp = yyvsp[-1].cp; }
break;
case 245:
#line 1635 "ns_parser.y"
{
		if (lookup_acl(yyvsp[-3].cp) != NULL) {
			parser_error(0, "can't redefine ACL '%s'", yyvsp[-3].cp);
		} else
			define_acl(yyvsp[-3].cp, yyvsp[-1].iml);
		(void)freestr(yyvsp[-3].cp);
	}
break;
case 246:
#line 1649 "ns_parser.y"
{
		int sym_type;
		symbol_value value;
		char *zone_name;

		if (!seen_options && !logged_options_error) {
			parser_error(0,
             "no options statement before first zone; using previous/default");
			logged_options_error = 1;
		}
		sym_type = SYM_ZONE | (yyvsp[0].num & 0xffff);
		value.pointer = NULL;
		zone_name = canonical_name(yyvsp[-1].cp);
		if (zone_name == NULL) {
			parser_error(0, "can't make zone name '%s' canonical",
				     yyvsp[-1].cp);
			should_install = 0;
			zone_name = savestr("__bad_zone__", 1);
		} else {
			if (lookup_symbol(symtab, zone_name, sym_type, NULL)) {
				should_install = 0;
				parser_error(0,
					"cannot redefine zone '%s' class %s",
					     *zone_name ? zone_name : ".",
					     p_class(yyvsp[0].num));
			} else {
				should_install = 1;
				define_symbol(symtab, zone_name, sym_type,
					      value, 0);
			}
		}
		(void)freestr(yyvsp[-1].cp);
		current_zone = begin_zone(zone_name, yyvsp[0].num); 
	}
break;
case 247:
#line 1684 "ns_parser.y"
{
		end_zone(current_zone, should_install);
	}
break;
case 250:
#line 1694 "ns_parser.y"
{
		yyval.num = C_IN;
	}
break;
case 251:
#line 1698 "ns_parser.y"
{
		symbol_value value;

		if (lookup_symbol(constants, yyvsp[0].cp, SYM_CLASS, &value))
			yyval.num = value.integer;
		else {
			/* the zone validator will give the error */
			yyval.num = C_NONE;
		}
		(void)freestr(yyvsp[0].cp);
	}
break;
case 252:
#line 1712 "ns_parser.y"
{
		yyval.s_int = Z_MASTER;
	}
break;
case 253:
#line 1716 "ns_parser.y"
{
		yyval.s_int = Z_SLAVE;
	}
break;
case 254:
#line 1720 "ns_parser.y"
{
		yyval.s_int = Z_HINT;
	}
break;
case 255:
#line 1724 "ns_parser.y"
{
		yyval.s_int = Z_STUB;
	}
break;
case 256:
#line 1728 "ns_parser.y"
{
		yyval.s_int = Z_FORWARD;
	}
break;
case 259:
#line 1738 "ns_parser.y"
{
		if (!set_zone_type(current_zone, yyvsp[0].s_int))
			parser_warning(0, "zone type already set; skipping");
	}
break;
case 260:
#line 1743 "ns_parser.y"
{
		if (!set_zone_filename(current_zone, yyvsp[0].cp))
			parser_warning(0,
				       "zone filename already set; skipping");
	}
break;
case 261:
#line 1749 "ns_parser.y"
{
		if (!set_zone_ixfr_file(current_zone, yyvsp[0].cp))
			parser_warning(0,
				       "zone ixfr data base already set; skipping");
	}
break;
case 262:
#line 1755 "ns_parser.y"
{
		if (!set_zone_ixfr_tmp(current_zone, yyvsp[0].cp))
			parser_warning(0,
				       "zone ixfr temp filename already set; skipping");
	}
break;
case 263:
#line 1761 "ns_parser.y"
{
		set_zone_master_port(current_zone, yyvsp[-3].us_int);
	}
break;
case 264:
#line 1765 "ns_parser.y"
{
		set_zone_transfer_source(current_zone, sstosa(&yyvsp[0].ip_addr));
	}
break;
case 265:
#line 1769 "ns_parser.y"
{
		set_zone_transfer_source_v6(current_zone, sstosa(&yyvsp[0].ip_addr));
	}
break;
case 266:
#line 1773 "ns_parser.y"
{
		if (!set_zone_checknames(current_zone, (enum severity)yyvsp[0].s_int))
			parser_warning(0,
	                              "zone checknames already set; skipping");
	}
break;
case 267:
#line 1779 "ns_parser.y"
{
		if (!set_zone_update_acl(current_zone, yyvsp[-1].iml))
			parser_warning(0,
				      "zone update acl already set; skipping");
	}
break;
case 268:
#line 1785 "ns_parser.y"
{
		if (!set_zone_query_acl(current_zone, yyvsp[-1].iml))
			parser_warning(0,
				      "zone query acl already set; skipping");
	}
break;
case 269:
#line 1791 "ns_parser.y"
{
		if (!set_zone_transfer_acl(current_zone, yyvsp[-1].iml))
			parser_warning(0,
				    "zone transfer acl already set; skipping");
	}
break;
case 271:
#line 1798 "ns_parser.y"
{
		struct zoneinfo *zp = current_zone.opaque;
		if (zp->z_fwdtab) {
                	free_forwarders(zp->z_fwdtab);
			zp->z_fwdtab = NULL;
		}

	}
break;
case 273:
#line 1808 "ns_parser.y"
{
		if (!set_zone_transfer_time_in(current_zone, yyvsp[0].num*60))
			parser_warning(0,
		       "zone max transfer time (in) already set; skipping");
	}
break;
case 274:
#line 1814 "ns_parser.y"
{
		set_zone_max_log_size_ixfr(current_zone, yyvsp[0].ul_int);
        }
break;
case 275:
#line 1818 "ns_parser.y"
{
		set_zone_notify(current_zone, notify_explicit);
	}
break;
case 276:
#line 1822 "ns_parser.y"
{
		if (yyvsp[0].num)
			set_zone_notify(current_zone, notify_yes);
		else
			set_zone_notify(current_zone, notify_no);
	}
break;
case 277:
#line 1829 "ns_parser.y"
{
		set_zone_maintain_ixfr_base(current_zone, yyvsp[0].num);
	}
break;
case 278:
#line 1833 "ns_parser.y"
{
		/* flags proto alg key */
		set_zone_pubkey(current_zone, yyvsp[-3].num, yyvsp[-2].num, yyvsp[-1].num, yyvsp[0].cp);
	}
break;
case 279:
#line 1838 "ns_parser.y"
{
		/* flags proto alg key */
		char *endp;
		int flags = (int) strtol(yyvsp[-3].cp, &endp, 0);
		if (*endp != '\0')
			ns_panic(ns_log_parser, 1,
				 "Invalid flags string: %s", yyvsp[-3].cp);
		set_zone_pubkey(current_zone, flags, yyvsp[-2].num, yyvsp[-1].num, yyvsp[0].cp);

	}
break;
case 281:
#line 1850 "ns_parser.y"
{
		 set_zone_dialup(current_zone, yyvsp[0].num);
	}
break;
case 283:
#line 1857 "ns_parser.y"
{
		/* nothing */
	}
break;
case 284:
#line 1861 "ns_parser.y"
{
		/* nothing */
	}
break;
case 285:
#line 1867 "ns_parser.y"
{
	  	add_zone_master(current_zone, sstosa(&yyvsp[0].ip_addr), NULL);
	}
break;
case 286:
#line 1871 "ns_parser.y"
{
		add_zone_master(current_zone, sstosa(&yyvsp[-2].ip_addr), yyvsp[0].keyi);
	}
break;
case 289:
#line 1881 "ns_parser.y"
{
		/* nothing */
	}
break;
case 290:
#line 1885 "ns_parser.y"
{
		/* nothing */
	}
break;
case 291:
#line 1891 "ns_parser.y"
{
	  	add_zone_notify(current_zone, sstosa(&yyvsp[0].ip_addr));
	}
break;
case 292:
#line 1897 "ns_parser.y"
{
		set_zone_boolean_option(current_zone, OPTION_FORWARD_ONLY, 1);
	}
break;
case 293:
#line 1901 "ns_parser.y"
{
		set_zone_boolean_option(current_zone, OPTION_FORWARD_ONLY, 0);
	}
break;
case 294:
#line 1907 "ns_parser.y"
{
		set_zone_forward(current_zone);
	}
break;
case 296:
#line 1914 "ns_parser.y"
{
		/* nothing */
	}
break;
case 297:
#line 1918 "ns_parser.y"
{
		/* nothing */
	}
break;
case 298:
#line 1924 "ns_parser.y"
{
	  	add_zone_forwarder(current_zone, sstosa(&yyvsp[0].ip_addr));
	}
break;
case 299:
#line 1934 "ns_parser.y"
{
	}
break;
case 300:
#line 1938 "ns_parser.y"
{
		/* nothing */
	}
break;
case 301:
#line 1942 "ns_parser.y"
{
		/* nothing */
	}
break;
case 302:
#line 1947 "ns_parser.y"
{
		/* name flags proto alg key */
		set_trusted_key(yyvsp[-4].cp, yyvsp[-3].num, yyvsp[-2].num, yyvsp[-1].num, yyvsp[0].cp);
	}
break;
case 303:
#line 1952 "ns_parser.y"
{
		/* name flags proto alg key */
		char *endp;
		int flags = (int) strtol(yyvsp[-3].cp, &endp, 0);
		if (*endp != '\0')
			ns_panic(ns_log_parser, 1,
				 "Invalid flags string: %s", yyvsp[-3].cp);
		set_trusted_key(yyvsp[-4].cp, flags, yyvsp[-2].num, yyvsp[-1].num, yyvsp[0].cp);
	}
break;
case 304:
#line 1968 "ns_parser.y"
{
		if (yyvsp[0].num < 0 || yyvsp[0].num > 65535) {
		  	parser_warning(0, 
			  "invalid IP port number '%d'; setting port to 0",
			               (int)yyvsp[0].num);
			yyvsp[0].num = 0;
		} else
			yyval.us_int = htons(yyvsp[0].num);
	}
break;
#line 3399 "y.tab.c"
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yysslim && yygrowstack())
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;
yyoverflow:
    yyerror("yacc stack overflow");
yyabort:
    return (1);
yyaccept:
    return (0);
}
